@@ -53,7 +53,7 @@ static void decompress_error(struct terminal *term, struct cache_entry *ce, unsi
 	}
 	if (!display_error(term, TEXT_(T_DECOMPRESSION_ERROR), errp)) return;
 	u = display_url(term, ce->url, 1);
-	msg_box(term, getml(u, NULL), TEXT_(T_DECOMPRESSION_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_DECOMPRESSING_), u, TEXT_(T__wITH_), lib, cast_uchar ": ", msg, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+	msg_box(term, getml(u, NULL), TEXT_(T_DECOMPRESSION_ERROR), AL_CENTER, TEXT_(T_ERROR_DECOMPRESSING_), u, TEXT_(T__wITH_), lib, cast_uchar ": ", msg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 }
 
 #endif
@@ -68,7 +68,6 @@ static int decode_gzip(struct terminal *term, struct cache_entry *ce, int defl,
 	unsigned char old_zlib;
 	z_stream z;
 	off_t offset;
-	size_t header;
 	int r;
 	unsigned char *p;
 	struct fragment *f;
@@ -77,7 +76,6 @@ static int decode_gzip(struct terminal *term, struct cache_entry *ce, int defl,
 
 	retry_after_memory_error:
 	memory_error = 0;
-	header = 0;
 	decoder_memory_init(&p, &size, ce->length);
 	init_again:
 	err = 0;
@@ -126,10 +124,6 @@ static int decode_gzip(struct terminal *term, struct cache_entry *ce, int defl,
 		z.next_in = f->data;
 		z.avail_in = (unsigned)f->length;
 		if ((off_t)z.avail_in != f->length) overalloc();
-		if (header && !offset) {
-			z.next_in = (unsigned char *)z.next_in + header;
-			z.avail_in -= (unsigned)header;
-		}
 		repeat_frag:
 		if (skip_gzip_header == 2) {
 			if (z.avail_in < 8) goto finish;
@@ -763,7 +757,7 @@ int get_file_by_term(struct terminal *term, struct cache_entry *ce, unsigned cha
 		unsigned char *msg = get_err_msg(e);
 		if (display_error(term, TEXT_(T_ERROR), errp)) {
 			unsigned char *u = display_url(term, ce->url, 1);
-			msg_box(term, getml(u, NULL), TEXT_(T_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_LOADING), cast_uchar " ", u, cast_uchar ":\n\n", msg, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+			msg_box(term, getml(u, NULL), TEXT_(T_ERROR), AL_CENTER, TEXT_(T_ERROR_LOADING), cast_uchar " ", u, cast_uchar ":\n\n", msg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		}
 	}
 	if (list_empty(ce->frag)) return 1;
