@@ -189,6 +189,7 @@ void make_connection(struct connection *c, int port, int *sock, void (*func)(str
 	int socks_port = -1;
 	int as;
 	unsigned char *host;
+	size_t sl;
 	struct conn_info *b;
 	if (*c->socks_proxy) {
 		unsigned char *p = cast_uchar strchr(cast_const_char c->socks_proxy, '@');
@@ -217,7 +218,9 @@ void make_connection(struct connection *c, int port, int *sock, void (*func)(str
 	}
 	if (c->newconn)
 		internal("already making a connection");
-	b = mem_calloc(sizeof(struct conn_info) + strlen(cast_const_char host));
+	sl = strlen(cast_const_char host);
+	if (sl > MAXINT - sizeof(struct conn_info)) overalloc();
+	b = mem_calloc(sizeof(struct conn_info) + sl);
 	b->func = func;
 	b->sock = sock;
 	b->l.socks_port = socks_port;
