@@ -42,8 +42,7 @@ static void get_system_name(void)
 			else
 #endif
 			add_to_str(&str, &l, cast_uchar name.machine);
-			if (l >= MAX_STR_LEN) str[MAX_STR_LEN - 1] = 0;
-			strcpy(cast_char system_name, cast_const_char str);
+			safe_strncpy(system_name, str, MAX_STR_LEN);
 			mem_free(str);
 			return;
 		}
@@ -726,7 +725,7 @@ static unsigned char *get_home(int *n)
 		if (home) {
 			unsigned char *new_path;
 			ssize_t sz = cygwin_conv_path(CCP_WIN_A_TO_POSIX | CCP_ABSOLUTE, home, NULL, 0);
-			if (sz < 0)
+			if (sz < 0 || sz >= MAXINT)
 				goto skip_path_conv;
 			new_path = mem_alloc(sz);
 			sz = cygwin_conv_path(CCP_WIN_A_TO_POSIX | CCP_ABSOLUTE, home, new_path, sz);
@@ -792,7 +791,6 @@ skip_path_conv:;
 			home_links = stracpy(home);
 			goto add_dot_links;
 		}
-		mem_free(config_dir);
 	} else {
 		add_dot_links:
 #ifdef OPENVMS
@@ -846,6 +844,7 @@ skip_path_conv:;
 	failed:
 	mem_free(home_links);
 	mem_free(home);
+	if (config_dir) mem_free(config_dir);
 	return NULL;
 
 	home_ok:
@@ -858,6 +857,7 @@ skip_path_conv:;
 	}
 	add_to_strn(&home_links, cast_uchar "/");
 	mem_free(home);
+	if (config_dir) mem_free(config_dir);
 	return home_links;
 }
 
@@ -884,12 +884,12 @@ static int write_config_data(unsigned char *prefix, unsigned char *name, struct
 	config_file = stracpy(prefix);
 	if (!config_file) {
 		mem_free(c);
-		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", TEXT_(T_HOME_DIRECTORY_INACCESSIBLE), NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", TEXT_(T_HOME_DIRECTORY_INACCESSIBLE), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		return -1;
 	}
 	add_to_strn(&config_file, name);
 	if ((err = write_to_config_file(config_file, c, 1))) {
-		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", get_err_msg(err), NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", get_err_msg(err), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		mem_free(c);
 		mem_free(config_file);
 		return -1;
@@ -951,7 +951,10 @@ static unsigned char *dbl_rd(struct option *o, unsigned char *c)
 
 	if (!tok) return cast_uchar "Missing argument";
 
-	if (strlen(cast_const_char tok) >= 1000) return cast_uchar "Number is too long";
+	if (strlen(cast_const_char tok) >= 1000) {
+		mem_free(tok);
+		return cast_uchar "Number is too long";
+	}
 
 	d = strtod(cast_const_char tok, (char **)(void *)&end);
 
@@ -1331,9 +1334,12 @@ static struct list_head driver_params = { &driver_params, &driver_params };
 struct driver_param *get_driver_param(unsigned char *n)
 {
 	struct driver_param *dp;
+	size_t sl;
 	struct list_head *ldp;
 	foreach(struct driver_param, dp, ldp, driver_params) if (!casestrcmp(dp->name, n)) return dp;
-	dp = mem_calloc(sizeof(struct driver_param) + strlen(cast_const_char n));
+	sl = strlen(cast_const_char n);
+	if (sl > MAXINT - sizeof(struct driver_param)) overalloc();
+	dp = mem_calloc(sizeof(struct driver_param) + sl);
 	dp->kbd_codepage = -1;
 	strcpy(cast_char dp->name, cast_const_char n);
 	dp->shell = mem_calloc(1);
