@@ -474,7 +474,7 @@ static void enlarge_color_horizontal(unsigned short *in, int ix, int y,
 				in_idx = (int)((out_pos - 1) / (ox - 1));
 			else
 				in_idx = 0;
-			in_pos = in_idx * (ox - 1);
+			in_pos = (ulonglong)in_idx * (ox - 1);
 			in_end = in_pos + (ox - 1);
 			add_col_color(thread_col_buf, in + in_idx * 3, skip, y,
 				in_end - out_pos);
@@ -618,7 +618,7 @@ static void scale_color_horizontal(unsigned short *in, int ix, int y,
 			out_pos = (ulonglong)out_idx * ix;
 			out_end = out_pos + ix;
 			in_idx = (int)(out_pos / ox);
-			in_pos = in_idx * ox;
+			in_pos = (ulonglong)in_idx * ox;
 			do {
 				ulonglong in_begin, in_end;
 				in_begin = in_pos;
@@ -742,7 +742,7 @@ static void enlarge_color_vertical(unsigned short *in, int x, int iy,
 
 	n_threads = omp_start();
 #ifdef HAVE_OPENMP
-	use_omp = (!OPENMP_NONATOMIC | !(x & 1)) & (n_threads > 1) & (oy >= 24);
+	use_omp = (!OPENMP_NONATOMIC | !(x & 3)) & (n_threads > 1) & (oy >= 24);
 	if (!use_omp)
 		n_threads = 1;
 #endif
@@ -771,7 +771,7 @@ static void enlarge_color_vertical(unsigned short *in, int x, int iy,
 				in_idx = (int)((out_pos - 1) / (oy - 1));
 			else
 				in_idx = 0;
-			in_pos = in_idx * (oy - 1);
+			in_pos = (ulonglong)in_idx * (oy - 1);
 			in_end = in_pos + oy - 1;
 			add_row_color(thread_row_buf, in + in_idx * 3 * x, x,
 				in_end - out_pos);
@@ -887,7 +887,7 @@ static void scale_color_vertical(unsigned short *in, int x, int iy,
 	}
 	n_threads = omp_start();
 #ifdef HAVE_OPENMP
-	use_omp = (!OPENMP_NONATOMIC | !(x & 1)) & (n_threads > 1) & (oy >= 24);
+	use_omp = (!OPENMP_NONATOMIC | !(x & 3)) & (n_threads > 1) & (oy >= 24);
 	if (!use_omp)
 		n_threads = 1;
 #endif
@@ -914,7 +914,7 @@ static void scale_color_vertical(unsigned short *in, int x, int iy,
 			out_pos = (ulonglong)out_idx * iy;
 			out_end = out_pos + iy;
 			in_idx = (int)(out_pos / oy);
-			in_pos = in_idx * oy;
+			in_pos = (ulonglong)in_idx * oy;
 			do {
 				ulonglong in_begin, in_end;
 				in_begin = in_pos;
@@ -1789,7 +1789,7 @@ const unsigned char *png_data, int png_length, struct style *style)
 		for (x0=*x;x0;x0--){
 			/* Convolution */
 			c=(int)(((*i2ptr)*conv0)+i2ptr[-ix]*conv1+i2ptr[-1]*conv1+i2ptr[1]*conv1+i2ptr[ix]*conv1+(float)0.5);
-			if (((unsigned)c)>=256) c=c<0?0:255;
+			if (c & ~255) c=c<0?0:255;
 			*dptr=(unsigned char)c;
 			dptr++;
 			i2ptr++;
