@@ -163,8 +163,8 @@ static int do_external_lookup(unsigned char *name, unsigned char *host)
 		EINTRLOOP(rs, dup2(pi[1], 2));
 		if (rs == -1) _exit(1);
 		EINTRLOOP(rs, close(pi[1]));
-		EINTRLOOP(rs, execlp("host", "host", name, NULL));
-		EINTRLOOP(rs, execl("/usr/sbin/host", "host", name, NULL));
+		EINTRLOOP(rs, execlp("host", "host", cast_const_char name, (char *)NULL));
+		EINTRLOOP(rs, execl("/usr/sbin/host", "host", cast_const_char name, (char *)NULL));
 		_exit(1);
 	}
 	EINTRLOOP(rs, close(pi[1]));
@@ -502,6 +502,7 @@ static void free_dns_entry(struct dnsentry *dnsentry)
 static void end_dns_lookup(struct dnsquery *q, int a)
 {
 	struct dnsentry *dnsentry;
+	size_t sl;
 	void (*fn)(void *, int);
 	void *data;
 	/*debug("end lookup %s", q->name);*/
@@ -526,7 +527,9 @@ static void end_dns_lookup(struct dnsquery *q, int a)
 	if (a) goto e;
 	if (q->addr_preference != ipv6_options.addr_preference) goto e;
 	check_dns_cache_addr_preference();
-	dnsentry = mem_alloc(sizeof(struct dnsentry) + strlen(cast_const_char q->name));
+	sl = strlen(cast_const_char q->name);
+	if (sl > MAXINT - sizeof(struct dnsentry)) overalloc();
+	dnsentry = mem_alloc(sizeof(struct dnsentry) + sl);
 	strcpy(cast_char dnsentry->name, cast_const_char q->name);
 	memcpy(&dnsentry->addr, q->addr, sizeof(struct lookup_result));
 	dnsentry->absolute_time = get_absolute_time();
