@@ -251,7 +251,7 @@ void *debug_mem_alloc(unsigned char *file, int line, size_t size, int mayfail)
 	dos_poll_break();
 	debug_test_free(file, line);
 	if (!size) return DUMMY;
-	if (size > MAXINT) {
+	if (size > MAX_SIZE_T - L_D_S - RED_ZONE_INC) {
 		if (mayfail) return NULL;
 		overalloc_at(file, line);
 	}
@@ -299,7 +299,7 @@ void *debug_mem_calloc(unsigned char *file, int line, size_t size, int mayfail)
 	dos_poll_break();
 	debug_test_free(file, line);
 	if (!size) return DUMMY;
-	if (size > MAXINT) {
+	if (size > MAX_SIZE_T - L_D_S - RED_ZONE_INC) {
 		if (mayfail) return NULL;
 		overalloc_at(file, line);
 	}
@@ -395,7 +395,7 @@ void *debug_mem_realloc(unsigned char *file, int line, void *p, size_t size, int
 		debug_mem_free(file, line, p);
 		return DUMMY;
 	}
-	if (size > MAXINT) {
+	if (size > MAX_SIZE_T - L_D_S - RED_ZONE_INC) {
 		if (mayfail) return NULL;
 		overalloc_at(file, line);
 	}
@@ -477,10 +477,6 @@ void *mem_alloc_(size_t size, int mayfail)
 	dos_poll_break();
 	debug_test_free(NULL, 0);
 	if (!size) return DUMMY;
-	if (size > MAXINT) {
-		if (mayfail) return NULL;
-		overalloc();
-	}
 	retry:
 	if (!(p = heap_malloc(size))) {
 		if (out_of_memory_fl(0, !mayfail ? cast_uchar "malloc" : NULL, size, NULL, 0)) goto retry;
@@ -495,10 +491,6 @@ void *mem_calloc_(size_t size, int mayfail)
 	dos_poll_break();
 	debug_test_free(NULL, 0);
 	if (!size) return DUMMY;
-	if (size > MAXINT) {
-		if (mayfail) return NULL;
-		overalloc();
-	}
 	retry:
 	if (!(p = heap_calloc(size))) {
 		if (out_of_memory_fl(0, !mayfail ? cast_uchar "calloc" : NULL, size, NULL, 0)) goto retry;
@@ -532,10 +524,6 @@ void *mem_realloc_(void *p, size_t size, int mayfail)
 		mem_free(p);
 		return DUMMY;
 	}
-	if (size > MAXINT) {
-		if (mayfail) return NULL;
-		overalloc();
-	}
 	retry:
 	if (!(np = heap_realloc(p, size))) {
 		if (out_of_memory_fl(0, !mayfail ? cast_uchar "realloc" : NULL, size, NULL, 0)) goto retry;
