@@ -5,15 +5,31 @@
 
 #include "links.h"
 
+#if defined(HAVE_STRMODE) || defined(HAVE_LIBBSD)
+
+static void stat_mode(unsigned char **p, int *l, struct stat *stp)
+{
+	unsigned char bp[12] = "?--------- ";
+	if (stp)
+		strmode(stp->st_mode, cast_char bp);
+#ifndef FS_UNIX_RIGHTS
+	bp[1] = ' ';
+	bp[2] = 0;
+#endif
+	add_to_str(p, l, bp);
+}
+
+#else
+
 #ifdef FS_UNIX_RIGHTS
-static void setrwx(int m, unsigned char *p)
+static void setrwx(unsigned m, unsigned char *p)
 {
 	if (m & S_IRUSR) p[0] = 'r';
 	if (m & S_IWUSR) p[1] = 'w';
 	if (m & S_IXUSR) p[2] = 'x';
 }
 
-static void setst(int m, unsigned char *p)
+static void setst(unsigned m, unsigned char *p)
 {
 #ifdef S_ISUID
 	if (m & S_ISUID) {
@@ -67,7 +83,7 @@ static void stat_mode(unsigned char **p, int *l, struct stat *stp)
 	{
 		unsigned char rwx[10] = "---------";
 		if (stp) {
-			int mode = stp->st_mode;
+			unsigned mode = stp->st_mode;
 			setrwx(mode << 0, &rwx[0]);
 			setrwx(mode << 3, &rwx[3]);
 			setrwx(mode << 6, &rwx[6]);
@@ -79,6 +95,8 @@ static void stat_mode(unsigned char **p, int *l, struct stat *stp)
 	add_chr_to_str(p, l, ' ');
 }
 
+#endif
+
 
 static void stat_links(unsigned char **p, int *l, struct stat *stp)
 {
