@@ -183,6 +183,19 @@ char *strcpy(char *dst, const char *src)
 	return memcpy(dst, src, strlen(src) + 1);
 }
 #endif
+#ifndef HAVE_STRNCPY
+char *strncpy(char *dst, const char *src, size_t len)
+{
+	size_t sl = strlen(src);
+	if (sl >= len) {
+		memcpy(dst, src, len);
+	} else {
+		memcpy(dst, src, sl);
+		memset(dst + sl, 0, len - sl);
+	}
+	return dst;
+}
+#endif
 #ifndef HAVE_STRCHR
 char *strchr(const char *s, int c)
 {
