@@ -92,7 +92,7 @@ struct fb_fillrect {
 
 static int TTY = 0;
 
-static int fb_hgpm;
+static int fb_hgpm = -2;
 
 static int fb_console;
 
@@ -319,19 +319,20 @@ static void fb_key_in(struct itrm *p, unsigned char *ev_, int size)
 	if (size != sizeof(struct links_event)) return;
 	if (ev->ev == EV_ABORT) terminate_loop = 1;
 	if (ev->ev != EV_KBD) return;
-	if ((ev->y & KBD_ALT) && ev->x >= '0' && ev->x <= '9') {
+	if (ev->y & KBD_PASTE) goto skip;
+	if ((ev->y & (KBD_CTRL | KBD_ALT)) == KBD_ALT && ev->x >= '0' && ev->x <= '9') {
 		switch_virtual_device((ev->x - '1' + 10) % 10);
 		return;
 	}
-	if (!current_virtual_device) return;
 	if (!ev->y && ev->x == KBD_F5) fb_mouse_move(-3, 0);
 	else if (!ev->y && ev->x == KBD_F6) fb_mouse_move(0, 3);
 	else if (!ev->y && ev->x == KBD_F7) fb_mouse_move(0, -3);
 	else if (!ev->y && ev->x == KBD_F8) fb_mouse_move(3, 0);
 	else {
+skip:
 		if (g_kbd_codepage(&fb_driver) != utf8_table && ev->x >= 128 && ev->x <= 255)
 			if ((ev->x = cp2u(ev->x, g_kbd_codepage(&fb_driver))) == -1) return;
-		if (current_virtual_device->keyboard_handler) current_virtual_device->keyboard_handler(current_virtual_device, ev->x, ev->y);
+		if (current_virtual_device && current_virtual_device->keyboard_handler) current_virtual_device->keyboard_handler(current_virtual_device, ev->x, ev->y);
 	}
 }
 
@@ -944,10 +945,14 @@ static void unhandle_fb_mouse(void);
 static void fb_gpm_in(void *nic)
 {
 	struct links_event ev;
+	int g;
 	Gpm_Event gev;
 	again:
 	set_handlers(fb_hgpm, (void (*)(void *))NULL, (void (*)(void *))NULL, NULL);
-	if (Gpm_GetEvent(&gev) <= 0) {
+	save_gpm_signals();
+	g = Gpm_GetEvent(&gev);
+	restore_gpm_signals();
+	if (g <= 0) {
 		fb_hgpm = -1;
 		unhandle_fb_mouse();
 		return;
@@ -1035,12 +1040,16 @@ static void fb_gpm_in(void *nic)
 static int handle_fb_mouse(void)
 {
 	Gpm_Connect conn;
-	fb_hgpm = -1;
+	fb_hgpm = -2;
 	conn.eventMask = (unsigned short)~0U;
 	conn.defaultMask = gpm_smooth;
 	conn.minMod = 0;
 	conn.maxMod = (unsigned short)~0U;
-	if ((fb_hgpm = Gpm_Open(&conn, 0)) < 0) {
+	save_gpm_signals();
+	fb_hgpm = Gpm_Open(&conn, 0);
+	restore_gpm_signals();
+	if (fb_hgpm < 0) {
+		fb_hgpm = -2;
 		unhandle_fb_mouse();
 		return -1;
 	}
@@ -1061,7 +1070,12 @@ static int handle_fb_mouse(void)
 static void unhandle_fb_mouse(void)
 {
 	if (fb_hgpm >= 0) set_handlers(fb_hgpm, (void (*)(void *))NULL, (void (*)(void *))NULL, NULL);
-	Gpm_Close();
+	if (fb_hgpm >= -1) {
+		save_gpm_signals();
+		Gpm_Close();
+		restore_gpm_signals();
+	}
+	fb_hgpm = -2;
 #ifdef SIGTSTP
 	install_signal_handler(SIGTSTP, sig_tstp, NULL, 0);
 #endif
