@@ -233,54 +233,58 @@ void flush_pending_text_to_line(struct g_part *p)
 	p->text = NULL;
 }
 
-static void split_line_object(struct g_part *p, struct g_object_text *text, unsigned char *ptr)
+static void split_line_object(struct g_part *p, struct g_object *text_go, unsigned char *ptr)
 {
+	struct g_object_text *text_t = NULL;
 	struct g_object_text *t2;
 	struct g_object_line *l2;
+	size_t sl;
 	int n;
 	if (!ptr) {
-		if (p->line && p->line->n_entries && &text->goti.go == p->line->entries[p->line->n_entries - 1]) {
+		if (p->line && p->line->n_entries && text_go == p->line->entries[p->line->n_entries - 1]) {
 			flush_pending_line_to_obj(p, 0);
 			goto wwww;
 		}
 		t2 = NULL;
 		goto nt2;
 	}
-	if (par_format.align == AL_NO_BREAKABLE && text == p->text && strspn(cast_const_char ptr, cast_const_char " ") == strlen(cast_const_char ptr)) {
+	text_t = get_struct(text_go, struct g_object_text, goti.go);
+	if (par_format.align == AL_NO_BREAKABLE && text_t == p->text && strspn(cast_const_char ptr, cast_const_char " ") == strlen(cast_const_char ptr)) {
 		return;
 	}
 #ifdef DEBUG
-	if (ptr < text->text || ptr >= text->text + strlen(cast_const_char text->text))
-		internal("split_line_object: split point (%p) pointing out of object (%p,%lx)", ptr, text->text, (unsigned long)strlen(cast_const_char text->text));
+	if (ptr < text_t->text || ptr >= text_t->text + strlen(cast_const_char text_t->text))
+		internal("split_line_object: split point (%p) pointing out of object (%p,%lx)", ptr, text_t->text, (unsigned long)strlen(cast_const_char text_t->text));
 #endif
-	t2 = mem_calloc(sizeof(struct g_object_text) + strlen(cast_const_char ptr));
+	sl = strlen(cast_const_char ptr);
+	if (sl > MAXINT - sizeof(struct g_object_text)) overalloc();
+	t2 = mem_calloc(sizeof(struct g_object_text) + sl);
 	t2->goti.go.mouse_event = g_text_mouse;
 	t2->goti.go.draw = g_text_draw;
 	t2->goti.go.destruct = g_text_destruct;
 	t2->goti.go.get_list = NULL;
 	if (*ptr == ' ') {
-		strcpy(cast_char t2->text, cast_const_char(ptr + 1));
+		memcpy(t2->text, ptr + 1, sl);
 		*ptr = 0;
-		/*debug("split: (%s)(%s)", text->text, ptr + 1);*/
+		/*debug("split: (%s)(%s)", text_t->text, ptr + 1);*/
 	} else {
-		strcpy(cast_char t2->text, cast_const_char ptr);
+		memcpy(t2->text, ptr, sl + 1);
 		ptr[0] = '-';
 		ptr[1] = 0;
 	}
-	t2->goti.go.y = text->goti.go.y;
-	t2->style = g_clone_style(text->style);
-	t2->goti.link_num = text->goti.link_num;
-	t2->goti.link_order = text->goti.link_order;
-	text->goti.go.xw = g_text_width(text->style, text->text);
+	t2->goti.go.y = text_t->goti.go.y;
+	t2->style = g_clone_style(text_t->style);
+	t2->goti.link_num = text_t->goti.link_num;
+	t2->goti.link_order = text_t->goti.link_order;
+	text_t->goti.go.xw = g_text_width(text_t->style, text_t->text);
 	nt2:
-	if (p->line) for (n = 0; n < p->line->n_entries; n++) if (p->line->entries[n] == &text->goti.go) goto found;
-	if (text != p->text) {
+	if (p->line) for (n = 0; n < p->line->n_entries; n++) if (p->line->entries[n] == text_go) goto found;
+	if (!text_t || text_t != p->text) {
 		internal("split_line_object: bad wrap");
 		return;
 	}
 	if (0) {
 		int nn;
-		int qw;
 		found:
 		n = safe_add(n, !ptr);
 		l2 = mem_calloc(sizeof(struct g_object_line) + (p->line->n_entries - n) * sizeof(struct g_object_text *));
@@ -290,14 +294,15 @@ static void split_line_object(struct g_part *p, struct g_object_text *text, unsi
 		l2->go.get_list = g_line_get_list;
 		l2->bg = p->root->bg;
 		l2->n_entries = p->line->n_entries - n;
-		l2->entries[0] = &t2->goti.go;
+		if (ptr)
+			l2->entries[0] = &t2->goti.go;
 		memcpy(&l2->entries[!!ptr], p->line->entries + safe_add(n, !!ptr), (l2->n_entries - !!ptr) * sizeof(struct g_object_text *));
 		p->line->n_entries = safe_add(n, !!ptr);
 		flush_pending_line_to_obj(p, 0);
 		p->line = l2;
 		if (ptr) {
 			t2->goti.go.xw = g_text_width(t2->style, t2->text);
-			t2->goti.go.yw = text->goti.go.yw;
+			t2->goti.go.yw = text_t->goti.go.yw;
 			p->w.pos = 0;
 		}
 		for (nn = 0; nn < l2->n_entries; nn++) {
@@ -305,15 +310,17 @@ static void split_line_object(struct g_part *p, struct g_object_text *text, unsi
 			/*debug("a1: %d (%s)", l2->entries[nn]->xw, tt->text);*/
 		}
 		wwww:
-		qw = g_text_width(p->text->style, p->text->text);
-		if (p->text) p->w.pos = safe_add(p->w.pos, qw);
+		if (p->text) {
+			int qw = g_text_width(p->text->style, p->text->text);
+			p->w.pos = safe_add(p->w.pos, qw);
+		}
 		/*debug("%d", p->w.pos);*/
 	} else {
 		flush_pending_text_to_line(p);
 		flush_pending_line_to_obj(p, 0);
 		p->line = NULL;
 		t2->goti.go.xw = g_text_width(t2->style, t2->text);
-		t2->goti.go.yw = text->goti.go.yw;
+		t2->goti.go.yw = text_t->goti.go.yw;
 		p->text = t2;
 		p->pending_text_len = -1;
 		p->w.pos = t2->goti.go.xw;
@@ -603,6 +610,7 @@ static void *g_html_special(void *p_, int c, ...)
 	struct g_part *p = p_;
 	va_list l;
 	unsigned char *t;
+	size_t sl;
 	struct form_control *fc;
 	struct frameset_param *fsp;
 	struct frame_param *fp;
@@ -616,11 +624,13 @@ static void *g_html_special(void *p_, int c, ...)
 			t = va_arg(l, unsigned char *);
 			va_end(l);
 			/* not needed to convert %AB here because html_tag will be called anyway */
-			tag = mem_calloc(sizeof(struct g_object_tag) + strlen(cast_const_char t));
+			sl = strlen(cast_const_char t);
+			if (sl > MAXINT - sizeof(struct g_object_tag)) overalloc();
+			tag = mem_calloc(sizeof(struct g_object_tag) + sl);
 			tag->go.mouse_event = g_dummy_mouse;
 			tag->go.draw = g_dummy_draw;
 			tag->go.destruct = g_tag_destruct;
-			strcpy(cast_char tag->name, cast_const_char t);
+			memcpy(tag->name, t, sl + 1);
 			flush_pending_text_to_line(p);
 			add_object_to_line(p, &p->line, &tag->go);
 			break;
@@ -803,7 +813,7 @@ static void g_put_chars(void *p_, unsigned char *s, int l)
 		p->w.force_break = 0;
 		if (p->w.width < 0) p->w.width = 0;
 		if (!g_wrap_text(&p->w)) {
-			split_line_object(p, get_struct(p->w.last_wrap_obj, struct g_object_text, goti.go), p->w.last_wrap);
+			split_line_object(p, p->w.last_wrap_obj, p->w.last_wrap);
 		}
 	}
 	return;
