@@ -268,7 +268,7 @@ static void xpand_lines(struct part *p, int y)
 		if ((y ^ p->data->y) > p->data->y) {
 			unsigned s;
 			for (s = 1; s < (unsigned)y; s = s * 2 + 1) {
-				if (s >= MAXINT) overalloc();
+				if (s > MAXINT / sizeof(struct line)) overalloc();
 			}
 			p->data->data = mem_realloc(p->data->data, s * sizeof(struct line));
 		}
@@ -571,13 +571,16 @@ struct link *new_link(struct f_data *f)
 void html_tag(struct f_data *f, unsigned char *t, int x, int y)
 {
 	struct tag *tag;
+	size_t sl;
 	unsigned char *tt;
 	int ll;
 	if (!f) return;
 	tt = init_str();
 	ll = 0;
 	add_conv_str(&tt, &ll, t, (int)strlen(cast_const_char t), -2);
-	tag = mem_alloc(sizeof(struct tag) + strlen(cast_const_char tt));
+	sl = strlen(cast_const_char tt);
+	if (sl > MAXINT - sizeof(struct tag)) overalloc();
+	tag = mem_alloc(sizeof(struct tag) + sl);
 	tag->x = x;
 	tag->y = y;
 	strcpy(cast_char tag->name, cast_const_char tt);
@@ -649,6 +652,7 @@ static void put_chars(void *p_, unsigned char *c, int l)
 		bad_utf:
 		p->utf8_part_len = 0;
 		if (!l) return;
+		if ((unsigned)l > (unsigned)MAXINT / sizeof(char_t)) overalloc();
 		uni_c = mem_alloc(l * sizeof(char_t));
 		ll = 0;
 		cc = c;
@@ -975,8 +979,10 @@ static void *html_special(void *p_, int c, ...)
 			html_form_control(p, fc);
 			break;
 		case SP_TABLE:
+			va_end(l);
 			return convert_table;
 		case SP_USED:
+			va_end(l);
 			return (void *)(my_intptr_t)!!p->data;
 		case SP_FRAMESET:
 			fsp = va_arg(l, struct frameset_param *);
@@ -1508,7 +1514,7 @@ int get_search_data(struct f_data *f)
 {
 	if (f->search_pos) return 0;
 	if (get_srch(f)) return -1;
-	if ((unsigned)n_chr > MAXINT / sizeof(char_t) || (unsigned)n_pos > MAXINT / sizeof(struct search)) return -1;
+	if ((size_t)n_chr > MAX_SIZE_T / sizeof(char_t) || (size_t)n_pos > MAX_SIZE_T / sizeof(struct search)) return -1;
 	f->search_chr = mem_alloc_mayfail(n_chr * sizeof(char_t));
 	if (!f->search_chr) return -1;
 	f->search_pos = mem_alloc_mayfail(n_pos * sizeof(struct search));
