@@ -1608,8 +1608,8 @@ static void add_to_cell_sets(struct table_cell ****s, int **nn, int *n, struct r
 			int *nnn;
 			if ((unsigned)i > MAXINT / sizeof(struct table_cell ***) - 1) overalloc();
 			if ((unsigned)i > MAXINT / sizeof(int *) - 1) overalloc();
-			ns = mem_realloc(*s, (i + 1) * sizeof(struct table_cell ***));
-			nnn = mem_realloc(*nn, (i + 1) * sizeof(int *));
+			ns = mem_realloc(*s, (i + 1) * sizeof(struct table_cell **));
+			nnn = mem_realloc(*nn, (i + 1) * sizeof(int));
 			for (j = *n; j < i + 1; j++) ns[j] = DUMMY, nnn[j] = 0;
 			*s = ns;
 			*nn = nnn;
@@ -1894,7 +1894,11 @@ static void process_g_table(struct g_part *gp, struct table *t)
 			struct table_cell *c;
 			c = CELL(t, i, j);
 			if (c->root && c->tag) {
-				struct g_object_tag *tag = mem_calloc(sizeof(struct g_object_tag) + strlen(cast_const_char c->tag));
+				struct g_object_tag *tag;
+				size_t sl;
+				sl = strlen(cast_const_char c->tag);
+				if (sl > MAXINT - sizeof(struct g_object_tag)) overalloc();
+				tag = mem_calloc(sizeof(struct g_object_tag) + sl);
 				tag->go.mouse_event = g_dummy_mouse;
 				tag->go.draw = g_dummy_draw;
 				tag->go.destruct = g_tag_destruct;
