@@ -224,7 +224,7 @@ static void ssl_load_private_certificates(SSL_CTX *ctx)
 			mem_free(err_str);
 			mem_free(numfail_str);
 		} else {
-			msg_box(ses->term, getml(err_str, numfail_str, NULL), TEXT_(T_SSL_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_INITIALIZING_BUILT_IN_CERTIFICATES), ": ", err_str, ", ", TEXT_(T_FAILED), " ", numfail_str, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+			msg_box(ses->term, getml(err_str, numfail_str, NULL), TEXT_(T_SSL_ERROR), AL_CENTER, TEXT_(T_ERROR_INITIALIZING_BUILT_IN_CERTIFICATES), ": ", err_str, ", ", TEXT_(T_FAILED), " ", numfail_str, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		}
 	}
 }
@@ -685,6 +685,7 @@ SSL_SESSION *get_session_cache_entry(SSL_CTX *ctx, unsigned char *host, int port
 static void set_session_cache_entry(SSL_CTX *ctx, unsigned char *host, int port, SSL_SESSION *s)
 {
 	struct session_cache_entry *sce = find_session_cache_entry(ctx, host, port);
+	size_t sl;
 	if (sce) {
 		SSL_SESSION_free(sce->session);
 		if (s) {
@@ -697,7 +698,9 @@ static void set_session_cache_entry(SSL_CTX *ctx, unsigned char *host, int port,
 	}
 	if (!s)
 		return;
-	sce = mem_alloc(sizeof(struct session_cache_entry) + strlen(cast_const_char host));
+	sl = strlen(cast_const_char host);
+	if (sl > MAXINT - sizeof(sizeof(struct session_cache_entry))) return;
+	sce = mem_alloc(sizeof(struct session_cache_entry) + sl);
 	sce->absolute_time = get_absolute_time();
 	sce->ctx = ctx;
 	sce->session = s;
