@@ -9,6 +9,8 @@
 #define IN_BUF_SIZE		16
 
 #define USE_TWIN_MOUSE	1
+#define BRACKETED_PASTE	2
+
 #define TW_BUTT_LEFT	1
 #define TW_BUTT_MIDDLE	2
 #define TW_BUTT_RIGHT	4
@@ -125,10 +127,10 @@ void kbd_ctrl_c(void)
 }
 
 unsigned char init_seq[] = "\033)0\0337";
-unsigned char init_seq_x_mouse[] = "\033[?1000h\033[?1002h\033[?1005l\033[?1015l\033[?1006h";
+unsigned char init_seq_x_mouse[] = "\033[?1000h\033[?1002h\033[?1005l\033[?1015l\033[?1006h\033[?2004h";
 unsigned char init_seq_tw_mouse[] = "\033[?9h";
 unsigned char term_seq[] = "\033[2J\0338\r \b";
-unsigned char term_seq_x_mouse[] = "\033[?1000l\r       \r\033[?1002l\r       \r\033[?1006l\r       \r";
+unsigned char term_seq_x_mouse[] = "\033[?1000l\r       \r\033[?1002l\r       \r\033[?1006l\r       \r\033[?2004l\r       \r";
 unsigned char term_seq_tw_mouse[] = "\033[?9l";
 
 static void send_init_sequence(int h, int flags)
@@ -1090,10 +1092,12 @@ static int process_queue(struct itrm *itrm)
 					case 21: ev.x = KBD_F10; break;
 					case 23: ev.x = KBD_F11; break;
 					case 24: ev.x = KBD_F12; break;
+					case 200: itrm->flags |= BRACKETED_PASTE; break;
+					case 201: itrm->flags &= ~BRACKETED_PASTE; break;
 					} break;
 				case 'R':
 					resize_terminal();
-					break ;
+					break;
 				case 'M':
 				case '<':
 					if (c == 'M' && v == 5) {
@@ -1265,10 +1269,9 @@ do_alt:
 		itrm->qlen = el;
 	}
 	if (ev.x != -1) {
-		/*printf("event: %x %x\n", ev.x, ev.y);*/
+		if (itrm->flags & BRACKETED_PASTE && ev.ev == EV_KBD)
+			ev.y |= KBD_PASTE;
 		itrm->queue_event(itrm, (unsigned char *)&ev, sizeof(struct links_event));
-	} else {
-		/*printf("%d %d\n", itrm->qlen, el);fflush(stdout);*/
 	}
 	memmove(itrm->kqueue, itrm->kqueue + el, itrm->qlen -= el);
 	end:
