@@ -104,6 +104,8 @@ int get_default_language(void)
 	}
 
 	default_language = get_language_from_lang(cast_uchar "en");
+	if (default_language < 0)
+		internal("default language 'english' not found");
 	return default_language;
 }
 
@@ -170,7 +172,7 @@ static inline int is_direct_text(unsigned char *text)
 {
 /* Do not compare to dummyarray directly - thwart some misoptimizations */
 	unsigned char * volatile dm = dummyarray;
-	return !(text >= dm && text < dm + T__N_TEXTS);
+	return !((my_uintptr_t)text >= (my_uintptr_t)dm && (my_uintptr_t)text < (my_uintptr_t)(dm + T__N_TEXTS));
 }
 
 unsigned char *get_text_translation(unsigned char *text, struct terminal *term)
@@ -203,8 +205,8 @@ unsigned char *get_text_translation(unsigned char *text, struct terminal *term)
 		current_tra[text - dummyarray] = trn;
 	} else {
 		if (current_lang_charset && charset != current_lang_charset) {
-			current_tra = translation_array[language_idx][charset] = mem_alloc(sizeof (unsigned char **) * T__N_TEXTS);
-			memset(current_tra, 0, sizeof (unsigned char **) * T__N_TEXTS);
+			current_tra = translation_array[language_idx][charset] = mem_alloc(sizeof (unsigned char *) * T__N_TEXTS);
+			memset(current_tra, 0, sizeof (unsigned char *) * T__N_TEXTS);
 			goto tr;
 		}
 		if (!(trn = cast_uchar translations[language_idx].t[text - dummyarray].name)) {
