@@ -310,7 +310,7 @@ static void end_dump(struct object_request *r, void *p)
 		if (!casecmp(r->url, cast_uchar "file://", 7) && !o.hard_assume) {
 			o.assume_cp = get_commandline_charset();
 		}
-		if (!(fd->f_data = cached_format_html(fd, r, r->url, &o, NULL))) goto term_1;
+		if (!(fd->f_data = cached_format_html(fd, r, r->url, &o, NULL, 0))) goto term_1;
 		dump_to_file(fd->f_data, oh);
 		term_1:
 		reinit_f_data_c(fd);
