@@ -947,7 +947,7 @@ void close_fork_tty(void)
 	if (drv && drv->after_fork) drv->after_fork();
 #endif
 	if (terminal_pipe[1] != -1) EINTRLOOP(rs, close(terminal_pipe[1]));
-	if (s_unix_fd != -1) close(s_unix_fd);
+	if (s_unix_fd != -1) EINTRLOOP(rs, close(s_unix_fd));
 	foreach(struct terminal, t, lt, terminals) {
 		if (t->fdin > 0)
 			EINTRLOOP(rs, close(t->fdin));
@@ -1113,7 +1113,7 @@ unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog
 	unsigned char *new_path;
 	ssize_t sz;
 	sz = cygwin_conv_path(CCP_POSIX_TO_WIN_A | CCP_ABSOLUTE, file, NULL, 0);
-	if (sz < 0) return stracpy(file);
+	if (sz < 0 || sz >= MAXINT) return stracpy(file);
 	new_path = mem_alloc(sz);
 	sz = cygwin_conv_path(CCP_POSIX_TO_WIN_A | CCP_ABSOLUTE, file, new_path, sz);
 	if (sz < 0) {
@@ -1145,7 +1145,7 @@ unsigned char *os_conv_to_external_path(unsigned char *file, unsigned char *prog
 	unsigned char *new_path;
 	ssize_t sz, sz2;
 	sz = uwin_path(file, NULL, 0);
-	if (sz < 0) return stracpy(file);
+	if (sz < 0 || sz >= MAXINT) return stracpy(file);
 	new_path = mem_alloc(sz + 1);
 	sz2 = uwin_path(file, new_path, sz + 1);
 	if (sz2 < 0 || sz2 > sz) {
@@ -1463,14 +1463,14 @@ int exe(unsigned char *path, int fg)
 	}
 	/* UWin corrupts heap if we use threads and fork */
 	fd_lock();
-	pid = spawnl("/bin/sh", "/bin/sh", "-c", arg, NULL);
+	pid = spawnl("/bin/sh", "/bin/sh", "-c", arg, (char *)NULL);
 	fd_unlock();
 #else
 #if 1		/* spawn breaks mouse, do this only in graphics mode */
 	if (F && is_winnt()) {
 		/* spawn crashes on Win98 */
 		fd_lock();
-		spawnlp(_P_WAIT, cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, NULL);
+		spawnlp(_P_WAIT, cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, (char *)NULL);
 		fd_unlock();
 		/*FreeConsole();*/
 		goto free_ret;
@@ -1481,7 +1481,7 @@ int exe(unsigned char *path, int fg)
 		if (!pid) {
 	/* Win98 crashes if we spawn command.com and have some sockets open */
 			close_handles(0);
-			EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, NULL));
+			EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, (char *)NULL));
 			_exit(1);
 		}
 	}
@@ -1561,8 +1561,8 @@ int exe_on_background(unsigned char *path, unsigned char *del)
 		EINTRLOOP(pid, fork());
 		if (!pid) {
 			close_handles(0);
-			/*EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, NULL));*/
-			spawnlp(_P_DETACH, cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, NULL);
+			/*EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, (char *)NULL));*/
+			spawnlp(_P_DETACH, cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, (char *)NULL);
 			_exit(1);
 		}
 	}
@@ -1708,13 +1708,13 @@ static void call_resize(unsigned char *x1, int x, int y)
 #endif
 	snprintf(cast_char arg, (int)sizeof(arg), "mode %d,%d", x, y);
 #if defined(_UWIN) && !defined(__DMC__)
-	pid = spawnlp(x1, x1, "/c", arg, NULL);
+	pid = spawnlp(x1, x1, "/c", arg, (char *)NULL);
 #else
 #if 0		/* spawn breaks mouse, don't use this */
 	if (is_winnt()) {
 		/* spawn crashes on Win98 */
 		fd_lock();
-		spawnlp(_P_WAIT, x1, x1, "/c", arg, NULL);
+		spawnlp(_P_WAIT, x1, x1, "/c", arg, (char *)NULL);
 		fd_unlock();
 		return;
 	} else
@@ -1724,7 +1724,7 @@ static void call_resize(unsigned char *x1, int x, int y)
 		if (!pid) {
 	/* Win98 crashes if we spawn command.com and have some sockets open */
 			close_handles(1);
-			EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, NULL));
+			EINTRLOOP(rs, execlp(cast_const_char x1, cast_const_char x1, "/c", cast_const_char arg, (char *)NULL));
 			_exit(1);
 		}
 	}
@@ -1810,7 +1810,7 @@ int exe(unsigned char *path, int fg)
 	{
 		fd_lock();
 	}
-	pid = spawnlp(flags, shell, shell, "/c", path, NULL);
+	pid = spawnlp(flags, shell, shell, "/c", path, (char *)NULL);
 #ifdef G
 	if (F) {
 		if (old0 >= 0) EINTRLOOP(rs, dup2(old0, 0));
@@ -1926,7 +1926,7 @@ unsigned char *get_window_title(void)
 		win_title = stracpy(swData.szSwtitle);
 		if (swData.hwnd != NULLHANDLE && !os2_init_pm()) {
 			LONG len = WinQueryWindowTextLength(swData.hwnd);
-			if (len > 0) {
+			if (len > 0 && len < MAXINT) {
 				mem_free(win_title);
 				win_title = mem_alloc(len + 1);
 				win_title[0] = 0;
@@ -2657,13 +2657,7 @@ int get_input_handle(void)
 #endif /* defined(HAVE_BEGINTHREAD) && defined(HAVE__READ_KBD) */
 
 
-#ifdef USE_GPM
-
-struct gpm_mouse_spec {
-	int h;
-	void (*fn)(void *, unsigned char *, int);
-	void *data;
-};
+#if defined(USE_GPM) || defined(GRDRV_FB)
 
 /* GPM installs its own signal handlers and we don't want them */
 
@@ -2678,7 +2672,7 @@ static struct sigaction gpm_tstp;
 static unsigned char gpm_tstp_valid;
 #endif
 
-static void save_gpm_signals(void)
+void save_gpm_signals(void)
 {
 	sigset_t sig;
 	int rs;
@@ -2701,7 +2695,7 @@ static void save_gpm_signals(void)
 #endif
 }
 
-static void restore_gpm_signals(void)
+void restore_gpm_signals(void)
 {
 	int rs;
 #ifdef SIGWINCH
@@ -2716,6 +2710,16 @@ static void restore_gpm_signals(void)
 		EINTRLOOP(rs, do_sigprocmask(SIG_SETMASK, &gpm_sigset, NULL));
 }
 
+#endif
+
+#ifdef USE_GPM
+
+struct gpm_mouse_spec {
+	int h;
+	void (*fn)(void *, unsigned char *, int);
+	void *data;
+};
+
 static void gpm_mouse_in(void *gms_)
 {
 	struct gpm_mouse_spec *gms = (struct gpm_mouse_spec *)gms_;
@@ -2865,9 +2869,14 @@ int get_system_env(void)
 static void exec_new_links(struct terminal *term, unsigned char *xterm, unsigned char *exe, unsigned char *param)
 {
 	unsigned char *str;
-	str = mem_alloc(strlen(cast_const_char xterm) + 1 + strlen(cast_const_char exe) + 1 + strlen(cast_const_char param) + 1);
-	if (*xterm) sprintf(cast_char str, "%s %s %s", xterm, exe, param);
-	else sprintf(cast_char str, "%s %s", exe, param);
+	str = stracpy(cast_uchar "");
+	if (*xterm) {
+		add_to_strn(&str, xterm);
+		add_to_strn(&str, cast_uchar " ");
+	}
+	add_to_strn(&str, exe);
+	add_to_strn(&str, cast_uchar " ");
+	add_to_strn(&str, param);
 	exec_on_terminal(term, str, cast_uchar "", 2);
 	mem_free(str);
 }
@@ -2980,7 +2989,7 @@ static int open_in_new_g(struct terminal *term, unsigned char *exe, unsigned cha
 }
 #endif
 
-static struct {
+static const struct {
 	int env;
 	int (*open_window_fn)(struct terminal *term, unsigned char *, unsigned char *);
 	unsigned char *text;
@@ -3005,7 +3014,6 @@ static struct {
 #ifdef G
 	{ENV_G, open_in_new_g, TEXT_(T_WINDOW), TEXT_(T_HK_WINDOW)},
 #endif
-	{0, NULL, NULL, NULL}
 };
 
 struct open_in_new *get_open_in_new(int environment)
@@ -3015,12 +3023,12 @@ struct open_in_new *get_open_in_new(int environment)
 	int noin = 0;
 	if (anonymous) return NULL;
 	if (environment & ENV_G) environment = ENV_G;
-	for (i = 0; oinw[i].env; i++) if ((environment & oinw[i].env) == oinw[i].env) {
+	for (i = 0; i < (int)array_elements(oinw); i++) if ((environment & oinw[i].env) == oinw[i].env) {
 		if ((unsigned)noin > MAXINT / sizeof(struct open_in_new) - 2) overalloc();
 		oin = mem_realloc(oin, (noin + 2) * sizeof(struct open_in_new));
 		oin[noin].text = oinw[i].text;
 		oin[noin].hk = oinw[i].hk;
-		oin[noin].open_window_fn = oinw[i].open_window_fn;
+		oin[noin].open_window_fn = &oinw[i].open_window_fn;
 		noin++;
 		oin[noin].text = NULL;
 		oin[noin].hk = NULL;
