@@ -15,20 +15,13 @@
 #endif
 #endif
 
-#ifndef MAXLONG
-#ifdef LONG_MAX
-#define MAXLONG LONG_MAX
-#else
-#define MAXLONG ((long)((unsigned long)-1L >> 1))
-#endif
-#endif
+#define MAX_SIZE_T	((size_t)-1)
 
-#if defined(HAVE_STRTOLL) && !defined(MAXLLONG)
-#ifdef LLONG_MAX
-#define MAXLLONG LLONG_MAX
-#else
-#define MAXLLONG ((long long)((unsigned long long)-1L >> 1))
-#endif
+#ifdef HAVE_MAXINT_CONVERSION_BUG
+#undef MAXINT
+#define MAXINT		0x7FFFF000
+#undef MAX_SIZE_T
+#define MAX_SIZE_T	((size_t)-0x1000)
 #endif
 
 #if !defined(HAVE_ERRNO) && !defined(errno)
@@ -239,11 +232,6 @@ int dos_select(int n, fd_set *rs, fd_set *ws, fd_set *es, struct timeval *t, int
 #undef HAVE_HERROR
 #endif
 
-#ifdef HAVE_MAXINT_CONVERSION_BUG
-#undef MAXINT
-#define MAXINT 0x7FFFF000
-#endif
-
 #ifndef HAVE_SOCKLEN_T
 #ifdef OPENVMS
 #define socklen_t unsigned
