@@ -128,7 +128,7 @@ static inline void ReleaseCapture(void)
 
 #endif
 
-#ifdef WIN32
+#ifdef WIN
 
 /*#define UNICODE*/
 /*#define NO_STRICT*/
@@ -182,10 +182,10 @@ static HICON icon_small;
 #define XBUTTON2		0x0002
 #endif
 #ifndef GET_X_LPARAM
-#define GET_X_LPARAM(lp) 	((short)LOWORD(lp))
+#define GET_X_LPARAM(lp)	((short)LOWORD(lp))
 #endif
 #ifndef GET_Y_LPARAM
-#define GET_Y_LPARAM(lp) 	((short)HIWORD(lp))
+#define GET_Y_LPARAM(lp)	((short)HIWORD(lp))
 #endif
 
 static HANDLE winapi_semaphore;
@@ -246,7 +246,7 @@ static inline void pm_event_signal(void)
 #define PM_ALLOC_ZERO		1
 #define PM_ALLOC_MAYFAIL	2
 
-#if defined(WIN32) && !defined(USE_WIN32_HEAP)
+#if defined(WIN) && !defined(USE_WIN32_HEAP)
 
 /* space allocated with malloc may span multiple mapped areas
    and SetDIBitsToDevice doesn't like it. So use HeapAlloc/HeapFree */
@@ -301,15 +301,14 @@ static int icon_set;
 #define E_RESIZE	4
 
 struct pm_event {
-	struct pm_event *next;
-	struct pm_event *prev;
+	list_entry_1st
 	int type;
 	int x1, y1, x2, y2;
+	list_entry_last
 };
 
 struct pm_window {
-	struct pm_window *next;
-	struct pm_window *prev;
+	list_entry_1st
 	int x, y;
 	unsigned char in;
 	unsigned char minimized;
@@ -319,7 +318,7 @@ struct pm_window {
 	HPS ps;
 	HWND h;
 #endif
-#ifdef WIN32
+#ifdef WIN
 	HDC dc;
 	HPEN pen_orig;
 	HPEN pen_cache;
@@ -332,6 +331,7 @@ struct pm_window {
 	int button;
 	unsigned long lastpos;
 	struct list_head queue;
+	list_entry_last
 };
 
 #define WIN_HASH	64
@@ -397,7 +397,7 @@ static void pm_send_event(struct pm_window *win, int t, int x1, int y1, int x2,
 static void pm_send_mouse_event(struct pm_window *win, int x1, int y1, int b)
 {
 	if (!list_empty(win->queue)) {
-		struct pm_event *last = win->queue.next;
+		struct pm_event *last = list_struct(win->queue.next, struct pm_event);
 		if (last->type == E_MOUSE && last->x2 == b) {
 			last->x1 = x1;
 			last->y1 = y1;
@@ -410,8 +410,9 @@ static void pm_send_mouse_event(struct pm_window *win, int x1, int y1, int b)
 static void pm_cancel_event(struct pm_window *win, int t, struct pm_event **pev)
 {
 	struct pm_event *ev;
+	struct list_head *lev;
 	if (pev) *pev = NULL;
-	foreachback(ev, win->queue) if (ev->type == t) {
+	foreachback(struct pm_event, ev, lev, win->queue) if (ev->type == t) {
 		if (pev) *pev = ev;
 		else {
 			del_from_list(ev);
@@ -473,7 +474,7 @@ static int win_x(struct pm_window *win)
 #ifdef OS2
 	int x = (short)(win->lastpos & 0xffff);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	int x = GET_X_LPARAM(win->lastpos);
 #endif
 	if (x < 0) x = -1;
@@ -486,7 +487,7 @@ static int win_y(struct pm_window *win)
 #ifdef OS2
 	int y = (short)(win->y - (win->lastpos >> 16));
 #endif
-#ifdef WIN32
+#ifdef WIN
 	int y = GET_Y_LPARAM(win->lastpos);
 #endif
 	if (y < 0) y = -1;
@@ -500,7 +501,7 @@ static void pm_user_msg(unsigned long msg, void *mp1, void *mp2);
 static MRESULT EXPENTRY pm_window_proc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
 #define mouse_pos	((unsigned)mp1)
 #endif
-#ifdef WIN32
+#ifdef WIN
 static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2)
 #define mouse_pos	((unsigned long)mp2)
 #endif
@@ -535,7 +536,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 			debug_call(("T: WM_PAINT: WinValidateRect"));
 			WinValidateRect(hwnd, &ur, FALSE);
 #endif
-#ifdef WIN32
+#ifdef WIN
 			GetUpdateRect(hwnd, &ur, FALSE);
 			if (!GetClientRect(hwnd, &wr)) {
 				memset(&wr, 0, sizeof wr);
@@ -558,7 +559,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 				debug_call(("T: WM_PAINT: break"));
 				break;
 			}
-#ifdef WIN32
+#ifdef WIN
 			ur.yTop = win->y - ur.yTop;
 			ur.yBottom = win->y - ur.yBottom;
 #endif
@@ -619,7 +620,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 			return 0;
 		case WM_CHAR:
 			debug_call(("T: WM_CHAR"));
-#ifdef WIN32
+#ifdef WIN
 		case WM_SYSCHAR:
 		case WM_KEYDOWN:
 		case WM_SYSKEYDOWN:
@@ -678,7 +679,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 				}
 			}
 #endif
-#ifdef WIN32
+#ifdef WIN
 			k_fsflags = (long)mp2;
 			flags = 0;
 			if (k_fsflags & (1 << 29))
@@ -823,7 +824,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 			}
 			debug_call(("T: BUTTON_UP: break"));
 			break;
-#ifdef WIN32
+#ifdef WIN
 		case WM_XBUTTONDOWN:
 		case WM_XBUTTONDBLCLK:
 			if (HIWORD(mp1) == XBUTTON1)
@@ -881,7 +882,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 			pm_unlock();
 			debug_call(("T: WM_MOUSEMOVE: break"));
 			break;
-#ifdef WIN32
+#ifdef WIN
 		case WM_CAPTURECHANGED:
 			pm_lock();
 			if (!(win = pm_lookup_window(hwnd))) { pm_unlock(); break; }
@@ -931,13 +932,14 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 			debug_call(("T: VM_HSCROLL: break"));
 			break;
 #endif
-#ifdef WIN32
+#ifdef WIN
 		case WM_MOUSEWHEEL:
+		case 0x020e:
 			flags = HIWORD(mp1);
 			if (!flags) break;
 			pm_lock();
 			if (!(win = pm_lookup_window(hwnd))) { pm_unlock(); break; }
-			pm_send_event(win, E_MOUSE, win_x(win), win_y(win), (flags > 0 && flags < 0x8000 ? B_WHEELUP : B_WHEELDOWN) | B_MOVE, 0);
+			pm_send_event(win, E_MOUSE, win_x(win), win_y(win), (flags > 0 && flags < 0x8000 ? (msg == WM_MOUSEWHEEL ? B_WHEELUP : B_WHEELRIGHT) : (msg == WM_MOUSEWHEEL ? B_WHEELDOWN: B_WHEELLEFT)) | B_MOVE, 0);
 			pm_unlock();
 			break;
 #endif
@@ -948,7 +950,7 @@ static LRESULT CALLBACK pm_window_proc(HWND hwnd, UINT msg, WPARAM mp1, LPARAM m
 	debug_call(("T: WinDefWindowProc returned %p", ret));
 	return ret;
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (!unicode_supported)
 		return DefWindowProcA(hwnd, msg, mp1, mp2);
 	else
@@ -983,7 +985,7 @@ static void pm_user_msg(unsigned long msg, void *mp1, void *mp2)
 					error("WinSetWindowPos failed");
 			}
 #endif
-#ifdef WIN32
+#ifdef WIN
 			if (unicode_supported) {
 				win->hc = CreateWindowExW(WS_EX_APPWINDOW | WS_EX_LEFT, (void *)(unsigned long)window_class_atom, L"Links", WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, module_handle, NULL);
 			} else {
@@ -998,6 +1000,8 @@ static void pm_user_msg(unsigned long msg, void *mp1, void *mp2)
 					if (icon_small != NULL) SendMessageA(win->hc, WM_SETICON, ICON_SMALL, (LPARAM)icon_small);
 				}
 				ShowWindow(win->hc, SW_SHOW);
+				/*SetWindowPos(win->hc, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);*/
+				SetForegroundWindow(win->hc);
 			}
 #endif
 			debug_call(("T: pm_lock"));
@@ -1013,7 +1017,7 @@ static void pm_user_msg(unsigned long msg, void *mp1, void *mp2)
 			if (!WinDestroyWindow(win->h))
 				fatal_exit("WinDestroyWindow failed");
 #endif
-#ifdef WIN32
+#ifdef WIN
 			if (!DestroyWindow(win->hc))
 				fatal_exit("DestroyWindow failed");
 #endif
@@ -1029,7 +1033,7 @@ static void pm_user_msg(unsigned long msg, void *mp1, void *mp2)
 			debug_call(("T: WinSetWindowText"));
 			WinSetWindowText(win->h, mp2);
 #endif
-#ifdef WIN32
+#ifdef WIN
 			if (unicode_supported && unicode_title_supported)
 				SetWindowTextW(win->hc, mp2);
 			else
@@ -1054,11 +1058,11 @@ static void pm_send_msg(int msg, void *param, void *param2)
 	debug_call(("M: calling WinPostMsg(%d, %p, %p)", msg, param, param2));
 	while (!WinPostMsg(hwnd_hidden, msg, (MPARAM)param, (MPARAM)param2)) {
 		debug_call(("M: WinPostMsg failed: %lx", WinGetLastError(hab)));
-		sleep(1);
+		portable_sleep(1000);
 	}
 	debug_call(("M: WinPostMsg succeeded"));
 #endif
-#ifdef WIN32
+#ifdef WIN
 	BOOL r;
 	if (!GdiFlush())
 		{/*error("GdiFlush failed: %u", (unsigned)GetLastError());*/}
@@ -1070,7 +1074,7 @@ static void pm_send_msg(int msg, void *param, void *param2)
 	if (!r) {
 		DWORD err = GetLastError();
 		if (err == ERROR_NOT_ENOUGH_QUOTA) {
-			sleep(1);
+			portable_sleep(1000);
 			goto retry;
 		}
 		fatal_exit("PostMessage failed: %x", (unsigned)GetLastError());
@@ -1078,7 +1082,7 @@ static void pm_send_msg(int msg, void *param, void *param2)
 #endif
 	pm_event_wait();
 	debug_call(("M: pm_event_wait succeeded"));
-#ifdef WIN32
+#ifdef WIN
 	if (msg == MSG_SHUTDOWN_THREAD) {
 		if (pthread_join(pthread_handle, NULL))
 			fatal_exit("pthread_join failed");
@@ -1092,7 +1096,7 @@ static void pm_dispatcher(void *p)
 	QMSG msg;
 	HWND hwnd_hidden_h;
 #endif
-#ifdef WIN32
+#ifdef WIN
 	MSG msg;
 	HCURSOR hcursor;
 	WNDCLASSEXW class_w;
@@ -1140,7 +1144,7 @@ static void pm_dispatcher(void *p)
 	}
 	GpiCreateLogColorTable(hps_hidden, 0, LCOLF_RGB, 0, 0, NULL);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	pm_cp = get_windows_cp(0);
 
 	hcursor = LoadCursorA(0, (void *)IDC_ARROW);
@@ -1208,7 +1212,7 @@ static void pm_dispatcher(void *p)
 		debug_call(("T: WinGetMsg: %lx, %lx, %p, %p, %lx, %lx.%lx, %lx", msg.hwnd, msg.msg, msg.mp1, msg.mp2, msg.time, msg.ptl.x, msg.ptl.y, msg.reserved));
 		WinDispatchMsg(hab_disp, &msg);
 #endif
-#ifdef WIN32
+#ifdef WIN
 		if (!unicode_supported)
 			GetMessageA(&msg, NULL, 0, 0);
 		else
@@ -1238,7 +1242,7 @@ static void pm_dispatcher(void *p)
 			error("WinTerminate failed");
 	os2_fail0:
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (!DestroyWindow(hwnd_hidden))
 		error("DestroyWindow failed: %u", (unsigned)GetLastError());
 	win32_fail:
@@ -1247,7 +1251,7 @@ static void pm_dispatcher(void *p)
 	return;
 }
 
-#ifdef WIN32
+#ifdef WIN
 static void *pm_dispatcher_win32(void *p)
 {
 	pm_dispatcher(NULL);
@@ -1255,10 +1259,10 @@ static void *pm_dispatcher_win32(void *p)
 }
 #endif
 
-static void pm_pipe_error(void *p)
+static void pm_pipe_error(void)
 {
 	error("exception on pm pipe");
-	set_handlers(pm_pipe[0], NULL, NULL, NULL, NULL);
+	set_handlers(pm_pipe[0], NULL, NULL, NULL);
 }
 
 static void pm_handler(void *p)
@@ -1270,9 +1274,9 @@ static void pm_handler(void *p)
 	pm_lock();
 	debug_call(("M: pm_handler: pm_locked"));
 	if (!list_empty(pm_event_windows)) {
-		win = pm_event_windows.prev;
+		win = list_struct(pm_event_windows.prev, struct pm_window);
 		if (!list_empty(win->queue)) {
-			ev = win->queue.prev;
+			ev = list_struct(win->queue.prev, struct pm_event);
 			del_from_list(ev);
 		}
 		if (list_empty(win->queue)) {
@@ -1284,7 +1288,7 @@ static void pm_handler(void *p)
 		int rd;
 		debug_call(("M: pm_handler: read"));
 		EINTRLOOP(rd, read(pm_pipe[0], &c, 1));
-		if (rd != 1) pm_pipe_error(NULL);
+		if (rd != 1) pm_pipe_error();
 	}
 	debug_call(("M: pm_handler: pm_unlock"));
 	pm_unlock();
@@ -1327,7 +1331,7 @@ static void pm_handler(void *p)
 
 #endif
 
-#ifdef WIN32
+#ifdef WIN
 
 static unsigned char flush_in_progress = 0;
 
@@ -1357,22 +1361,28 @@ static unsigned char *pm_get_driver_param(void)
 	return NULL;
 }
 
+static unsigned char *pm_get_af_unix_name(void)
+{
+	return cast_uchar "";
+}
+
 #ifdef PM_SPAWN_SUBPROC
 
 static int pm_sin, pm_sout, pm_serr, pm_ip[2], pm_op[2], pm_ep[2];
 static int pm_cons_ok = 0;
 
-static void pm_setup_console(void)
+static void pm_setup_console(int undo)
 {
 	int rs;
+	if (pm_cons_ok != undo) return;
 	if (pm_cons_ok) goto fail9;
-	set_bin(1);
-	set_bin(2);
-	EINTRLOOP(pm_sin, dup(0));
+	setmode(1, O_BINARY);
+	setmode(2, O_BINARY);
+	pm_sin = c_dup(0);
 	if (pm_sin < 0) goto fail;
-	EINTRLOOP(pm_sout, dup(1));
+	pm_sout = c_dup(1);
 	if (pm_sout < 0) goto fail1;
-	EINTRLOOP(pm_serr, dup(2));
+	pm_serr = c_dup(2);
 	if (pm_serr < 0) goto fail2;
 	if (c_pipe(pm_ip)) goto fail3;
 	if (c_pipe(pm_op)) goto fail4;
@@ -1483,16 +1493,16 @@ static unsigned char *pm_spawn_subproc(void)
 	arg[g_argc] = NULL;
 	pm_child_pid = -1;
 	install_signal_handler(SIGCHLD, pm_sigcld, NULL, 1);
-	pm_setup_console();
+	pm_setup_console(0);
 	pm_child_pid = spawnvp(P_PM, path_to_exe, (char **)arg);
 	mem_free(arg);
 	if (pm_child_pid < 0) {
 		set_sigcld();
-		pm_setup_console();
+		pm_setup_console(1);
 		return cast_uchar "Unable to spawn subprocess.\n";
 	}
 	pm_do_console();
-	pm_setup_console();
+	pm_setup_console(1);
 	while (1)
 		EINTRLOOP(rs, select(1, NULL, NULL, NULL, NULL));
 
@@ -1524,7 +1534,7 @@ static unsigned char *pm_init_driver(unsigned char *param, unsigned char *displa
 		goto r0;
 	}
 #endif
-#ifdef WIN32
+#ifdef WIN
 	module_handle = GetModuleHandleA(NULL);
 	if (!module_handle) {
 		s = cast_uchar "Unable to get module handle.\n";
@@ -1552,7 +1562,7 @@ static unsigned char *pm_init_driver(unsigned char *param, unsigned char *displa
 		goto r4;
 	}
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (pthread_create(&pthread_handle, NULL, pm_dispatcher_win32, NULL)) {
 		s = cast_uchar "Could not start thread.\n";
 		goto r4;
@@ -1588,7 +1598,7 @@ static unsigned char *pm_init_driver(unsigned char *param, unsigned char *displa
 
 	icon = WinLoadPointer(HWND_DESKTOP, 0, 1);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	{
 		int bpp;
 
@@ -1615,7 +1625,7 @@ static unsigned char *pm_init_driver(unsigned char *param, unsigned char *displa
 	icon_small = LoadImage(module_handle, MAKEINTRESOURCE(1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
 #endif
 	icon_set = 0;
-	set_handlers(pm_pipe[0], pm_handler, NULL, pm_pipe_error, NULL);
+	set_handlers(pm_pipe[0], pm_handler, NULL, NULL);
 
 	return NULL;
 
@@ -1657,7 +1667,7 @@ static void pm_shutdown_driver(void)
 	if (DevCloseDC(hdc_mem) == DEV_ERROR)
 		error("DevCloseDC failed");
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (icon_small != NULL)
 		if (!DestroyIcon(icon_small))
 			error("DestroIcon failed: %u", (unsigned)GetLastError());
@@ -1669,8 +1679,7 @@ static void pm_shutdown_driver(void)
 #endif
 	pm_free(pm_bitmapinfo);
 	pm_lock_close();
-	set_handlers(pm_pipe[0], NULL, NULL, NULL, NULL);
-	EINTRLOOP(rs, close(pm_pipe[0]));
+	close_socket(&pm_pipe[0]);
 	EINTRLOOP(rs, close(pm_pipe[1]));
 	pm_event_close();
 	PM_UNFLUSH();
@@ -1724,7 +1733,7 @@ static HPOINTER pmshell_create_icon(void)
 
 #endif
 
-#ifdef WIN32
+#ifdef WIN
 
 static HICON win32_create_icon(int x, int y)
 {
@@ -1842,7 +1851,7 @@ static struct graphics_device *pm_init_device(void)
 #ifdef OS2
 		if (icon == NULLHANDLE) icon = pmshell_create_icon();
 #endif
-#ifdef WIN32
+#ifdef WIN
 		if (!icon_big) icon_big = win32_create_icon(32, 32);
 		if (!icon_small) icon_small = win32_create_icon(16, 16);
 #endif
@@ -1867,7 +1876,7 @@ static struct graphics_device *pm_init_device(void)
 		goto r2;
 	}
 #endif
-#ifdef WIN32
+#ifdef WIN
 	win->pen_orig = NULL;
 	win->pen_cache = NULL;
 	win->pen_cache_color = -1;
@@ -1889,7 +1898,7 @@ static struct graphics_device *pm_init_device(void)
 		memset(&wr, 0, sizeof wr);
 	}
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (!GetClientRect(win->hc, &wr)) {
 		memset(&wr, 0, sizeof wr);
 	}
@@ -1927,7 +1936,7 @@ static void pm_shutdown_device(struct graphics_device *dev)
 	if (!WinReleasePS(win->ps))
 		error("WinReleasePS failed");
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (win->pen_orig)
 		if (!SelectObject(win->dc, win->pen_orig))
 			error("SelectObject failed for original pen %u", (unsigned)GetLastError());
@@ -1938,7 +1947,7 @@ static void pm_shutdown_device(struct graphics_device *dev)
 #endif
 	debug_call(("M: pm_shutdown_device: pm_send_msg"));
 	pm_send_msg(MSG_DELETE_WINDOW, win, NULL);
-#ifdef WIN32
+#ifdef WIN
 	if (win->pen_cache)
 		if (!DeleteObject(win->pen_cache))
 			error("DeleteObject failed for pen cache %u", (unsigned)GetLastError());
@@ -1953,7 +1962,7 @@ static void pm_shutdown_device(struct graphics_device *dev)
 	pm_unlock();
 	debug_call(("M: pm_shutdown_device: free"));
 	while (!list_empty(win->queue)) {
-		struct pm_event *ev = win->queue.next;
+		struct pm_event *ev = list_struct(win->queue.next, struct pm_event);
 		del_from_list(ev);
 		free(ev);
 	}
@@ -1968,7 +1977,7 @@ static void pm_set_window_title(struct graphics_device *dev, unsigned char *titl
 {
 	unsigned char *text;
 	debug_call(("M: pm_set_window_title: start"));
-#if defined(WIN32)
+#ifdef WIN
 	if (unicode_supported && unicode_title_supported) {
 		int l = 0;
 		text = init_str();
@@ -1984,8 +1993,7 @@ static void pm_set_window_title(struct graphics_device *dev, unsigned char *titl
 	} else
 #endif
 	{
-		struct conv_table *ct = get_translation_table(utf8_table, pm_cp);
-		text = convert_string(ct, title, strlen(cast_const_char title), NULL);
+		text = convert(utf8_table, pm_cp, title, NULL);
 		clr_white(text);
 		if (strlen(cast_const_char text) > MAX_TITLE_SIZE) text[MAX_TITLE_SIZE] = 0;
 	}
@@ -2187,7 +2195,7 @@ static void pm_draw_bitmap(struct graphics_device *dev, struct bitmap *bmp, int
 		}
 	}
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (!bmp->data)
 		return;
 	pm_bitmapinfo->bmiHeader.biWidth = bmp->x;
@@ -2203,7 +2211,7 @@ static long pm_get_color(int rgb)
 #ifdef OS2
 	return rgb & 0xffffff;
 #endif
-#ifdef WIN32
+#ifdef WIN
 	COLORREF c, d;
 	c = ((rgb & 0xff) << 16) | (rgb & 0xff00) | ((rgb & 0xff0000) >> 16);
 	d = GetNearestColor(screen_dc, c);
@@ -2212,7 +2220,7 @@ static long pm_get_color(int rgb)
 #endif
 }
 
-#ifdef WIN32
+#ifdef WIN
 
 static int win32_select_brush(struct pm_window *win, int color)
 {
@@ -2279,7 +2287,7 @@ static void pm_fill_area(struct graphics_device *dev, int x1, int y1, int x2, in
 	r.yTop = pm_win(dev)->y - r.yTop;
 	WinFillRect(pm_win(dev)->ps, &r, color);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (win32_select_brush(pm_win(dev), color))
 		return;
 	if (!FillRect(pm_win(dev)->dc, &r, pm_win(dev)->brush_cache))
@@ -2309,7 +2317,7 @@ static void pm_draw_hline(struct graphics_device *dev, int x1, int y, int x2, lo
 	p.x = x2 - 1;
 	GpiLine(ps, &p);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (win32_select_pen(pm_win(dev), color))
 		return;
 	if (!MoveToEx(pm_win(dev)->dc, x1, y, NULL)) {
@@ -2342,7 +2350,7 @@ static void pm_draw_vline(struct graphics_device *dev, int x, int y1, int y2, lo
 	p.y = pm_win(dev)->y - y2;
 	GpiLine(ps, &p);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (win32_select_pen(pm_win(dev), color))
 		return;
 	if (!MoveToEx(pm_win(dev)->dc, x, y1, NULL)) {
@@ -2388,7 +2396,7 @@ static int pm_hscroll(struct graphics_device *dev, struct rect_set **ignore, int
 	r.yTop = pm_win(dev)->y - r.yTop;
 	WinScrollWindow(pm_win(dev)->hc, sc, 0, &r, &r, NULLHANDLE, NULL, SW_INVALIDATERGN);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (ScrollWindowEx(pm_win(dev)->hc, sc, 0, &r, &r, NULL, NULL, SW_INVALIDATE) == ERROR)
 		error("ScrollWindowEx failed %u", (unsigned)GetLastError());
 #endif
@@ -2432,7 +2440,7 @@ static int pm_vscroll(struct graphics_device *dev, struct rect_set **ignore, int
 	r.yTop = pm_win(dev)->y - r.yTop;
 	WinScrollWindow(pm_win(dev)->hc, 0, -sc, &r, &r, NULLHANDLE, NULL, SW_INVALIDATERGN);
 #endif
-#ifdef WIN32
+#ifdef WIN
 	if (ScrollWindowEx(pm_win(dev)->hc, 0, sc, &r, &r, NULL, NULL, SW_INVALIDATE) == ERROR)
 		error("ScrollWindowEx failed %u", (unsigned)GetLastError());
 #endif
@@ -2443,19 +2451,32 @@ static int pm_vscroll(struct graphics_device *dev, struct rect_set **ignore, int
 	return 0;
 }
 
+#ifdef OS2
+#define pm_flush	NULL
+#endif
+#ifdef WIN
+static void pm_flush(struct graphics_device *dev)
+{
+	PM_UNFLUSH();
+	pm_do_flush(NULL);
+}
+#endif
+
 struct graphics_driver pmshell_driver = {
 #ifdef OS2
 	cast_uchar "pmshell",
 #endif
-#ifdef WIN32
+#ifdef WIN
 	cast_uchar "windows",
 #endif
 	pm_init_driver,
 	pm_init_device,
 	pm_shutdown_device,
 	pm_shutdown_driver,
-	dummy_emergency_shutdown,
+	NULL,
+	NULL,
 	pm_get_driver_param,
+	pm_get_af_unix_name,
 	NULL,
 	NULL,
 	pm_get_empty_bitmap,
@@ -2471,6 +2492,7 @@ struct graphics_driver pmshell_driver = {
 	pm_hscroll,
 	pm_vscroll,
 	generic_set_clip_area,
+	pm_flush,
 	dummy_block,
 	dummy_unblock,
 	pm_set_window_title,
@@ -2479,7 +2501,7 @@ struct graphics_driver pmshell_driver = {
 	NULL, /* get_clipboard_text */
 	0,			/* depth */
 	0, 0,			/* x, y */
-	0,			/* flags */
+	GD_UNICODE_KEYS,	/* flags */
 	0,			/* codepage */
 	NULL,			/* shell */
 };
