@@ -699,13 +699,15 @@ unsigned char *get_proxy_string(unsigned char *url)
 
 unsigned char *get_proxy(unsigned char *url)
 {
-	size_t l = strlen(cast_const_char url);
 	unsigned char *proxy = get_proxy_string(url);
 	unsigned char *u;
-	u = mem_alloc(l + 1 + (proxy ? strlen(cast_const_char proxy) + 9 : 0));
-	if (proxy) strcpy(cast_char u, "proxy://"), strcat(cast_char u, cast_const_char proxy), strcat(cast_char u, "/");
-	else *u = 0;
-	strcat(cast_char u, cast_const_char url);
+	u = stracpy(cast_uchar "");
+	if (proxy) {
+		add_to_strn(&u, cast_uchar "proxy://");
+		add_to_strn(&u, proxy);
+		add_to_strn(&u, cast_uchar "/");
+	}
+	add_to_strn(&u, url);
 	return u;
 }
 
@@ -1096,11 +1098,14 @@ void add_blacklist_entry(unsigned char *host, int flags)
 {
 	struct blacklist_entry *b;
 	struct list_head *lb;
+	size_t sl;
 	foreach(struct blacklist_entry, b, lb, blacklist) if (!casestrcmp(host, b->host)) {
 		b->flags |= flags;
 		return;
 	}
-	b = mem_alloc(sizeof(struct blacklist_entry) + strlen(cast_const_char host));
+	sl = strlen(cast_const_char host);
+	if (sl > MAXINT - sizeof(struct blacklist_entry)) overalloc();
+	b = mem_alloc(sizeof(struct blacklist_entry) + sl);
 	b->flags = flags;
 	strcpy(cast_char b->host, cast_const_char host);
 	add_to_list(blacklist, b);
