@@ -601,16 +601,18 @@ void *get_handler_data(int fd)
 
 void set_handlers_file_line(int fd, void (*read_func)(void *), void (*write_func)(void *), void *data)
 {
+	if (fd < 0)
+		goto invl;
 #ifdef DEBUG
 	{
 		struct stat st;
-		if (fstat(fd, &st) == -1 && errno == EBADF) {
+		int rs;
+		EINTRLOOP(rs, fstat(fd, &st));
+		if (rs == -1 && errno == EBADF) {
 			goto invl;
 		}
 	}
 #endif
-	if (fd < 0)
-		goto invl;
 #if defined(USE_POLL) && defined(USE_LIBEVENT)
 	if (!event_enabled)
 #endif
@@ -619,6 +621,7 @@ void set_handlers_file_line(int fd, void (*read_func)(void *), void (*write_func
 			return;
 		}
 	if (fd >= n_threads) {
+		if ((unsigned)fd > (unsigned)MAXINT / sizeof(struct thread) - 1) overalloc();
 		threads = mem_realloc(threads, (fd + 1) * sizeof(struct thread));
 		memset(threads + n_threads, 0, (fd + 1 - n_threads) * sizeof(struct thread));
 		n_threads = fd + 1;
