@@ -128,6 +128,7 @@ int get_error_from_errno(int errn)
 unsigned char *get_err_msg(int state)
 {
 	unsigned char *e;
+	size_t sl;
 	struct strerror_val *s;
 	struct list_head *ls;
 	if ((state >= S_MAX && state <= S__OK) || state >= S_WAIT) {
@@ -144,7 +145,9 @@ unsigned char *get_err_msg(int state)
 	goto unk;
 have_error:
 	foreach(struct strerror_val, s, ls, strerror_buf) if (!strcmp(cast_const_char s->msg, cast_const_char e)) return s->msg;
-	s = mem_alloc(sizeof(struct strerror_val) + strlen(cast_const_char e));
+	sl = strlen(cast_const_char e);
+	if (sl > MAXINT - sizeof(struct strerror_val)) overalloc();
+	s = mem_alloc(sizeof(struct strerror_val) + sl);
 	strcpy(cast_char s->msg, cast_const_char e);
 	add_to_list(strerror_buf, s);
 	return s->msg;
@@ -321,8 +324,7 @@ void print_error_dialog(struct session *ses, struct status *stat, unsigned char
 {
 	unsigned char *t = get_err_msg(stat->state);
 	unsigned char *u = display_url(ses->term, url, 1);
-	if (!t) return;
-	msg_box(ses->term, getml(u, NULL), TEXT_(T_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_LOADING), cast_uchar " ", u, cast_uchar ":\n\n", t, NULL, ses, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC /*, get_text_translation("Retry"), NULL, 0 !!! FIXME: retry */);
+	msg_box(ses->term, getml(u, NULL), TEXT_(T_ERROR), AL_CENTER, TEXT_(T_ERROR_LOADING), cast_uchar " ", u, cast_uchar ":\n\n", t, MSG_BOX_END, (void *)ses, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC /*, get_text_translation("Retry"), NULL, 0 !!! FIXME: retry */);
 }
 
 static inline unsigned char hx(int a)
@@ -650,8 +652,10 @@ void download_window_function(struct dialog_data *dlg)
 	mem_free(m);
 }
 
-void display_download(struct terminal *term, struct download *down, struct session *ses)
+void display_download(struct terminal *term, void *down_, void *ses_)
 {
+	struct download *down = (struct download *)down_;
+	struct session *ses = (struct session *)ses_;
 	struct dialog *dlg;
 	struct download *dd;
 	struct list_head *ldd;
@@ -810,7 +814,7 @@ static void download_file_error(struct download *down, int err)
 	if (get_download_ses(down)) {
 		unsigned char *emsg = stracpy(err ? cast_uchar strerror(err) : cast_uchar "Zero returned");
 		unsigned char *msg = stracpy(down->file);
-		msg_box(get_download_ses(down)->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_COULD_NOT_WRITE_TO_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+		msg_box(get_download_ses(down)->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_WRITE_TO_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 	}
 }
 
@@ -962,10 +966,8 @@ have_frag:
 		}
 		if (stat->state != S__OK) {
 			unsigned char *t = get_err_msg(stat->state);
-			if (t) {
-				unsigned char *tt = display_url(get_download_ses(down)->term, down->url, 1);
-				msg_box(get_download_ses(down)->term, getml(tt, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_DOWNLOADING), cast_uchar " ", tt, cast_uchar ":\n\n", t, NULL, get_download_ses(down), 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC /*, TEXT_(T_RETRY), NULL, 0 !!! FIXME: retry */);
-			}
+			unsigned char *tt = display_url(get_download_ses(down)->term, down->url, 1);
+			msg_box(get_download_ses(down)->term, getml(tt, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_ERROR_DOWNLOADING), cast_uchar " ", tt, cast_uchar ":\n\n", t, MSG_BOX_END, (void *)get_download_ses(down), 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC /*, TEXT_(T_RETRY), NULL, 0 !!! FIXME: retry */);
 		} else {
 			if (close_download_file(down)) {
 				download_file_error(down, errno);
@@ -1072,7 +1074,7 @@ int create_download_file(struct session *ses, unsigned char *cwd, unsigned char
 		if (!ses) goto x;
 		msg = stracpy(file);
 		msge = stracpy(cast_uchar strerror(errn));
-		msg_box(ses->term, getml(msg, msge, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_COULD_NOT_CREATE_FILE), cast_uchar " ", msg, cast_uchar ": ", msge, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+		msg_box(ses->term, getml(msg, msge, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_CREATE_FILE), cast_uchar " ", msg, cast_uchar ": ", msge, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		goto x;
 	}
 	x:
@@ -1109,14 +1111,14 @@ static int create_or_append_download_file(struct session *ses, unsigned char *cw
 		if (ls == (off_t)-1) {
 			unsigned char *emsg = stracpy(cast_uchar strerror(errno));
 			unsigned char *msg = stracpy(*xl_file);
-			msg_box(ses->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_ERROR_CALLING_LSEEK_ON_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+			msg_box(ses->term, getml(msg, emsg, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_ERROR_CALLING_LSEEK_ON_FILE), cast_uchar " ", msg, cast_uchar ": ", emsg, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 			goto err_close;
 		}
 		if ((off_t)(0UL + *last_pos + ls) < 0 ||
 		    (off_t)(0UL + *last_pos + ls) < *last_pos) {
 			unsigned char *msg1 = stracpy(fi);
 			unsigned char *msg2 = stracpy(*xl_file);
-			msg_box(ses->term, getml(msg1, msg2, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_TOO_LARGE_FILE_SEQUENCE), cast_uchar " ", msg1, cast_uchar " - ", msg2, NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+			msg_box(ses->term, getml(msg1, msg2, NULL), TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_TOO_LARGE_FILE_SEQUENCE), cast_uchar " ", msg1, cast_uchar " - ", msg2, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 			goto err_close;
 		}
 		*last_pos += ls;
@@ -1515,7 +1517,7 @@ static void calculate_scrollbars(struct f_data_c *fd, struct f_data *f)
 	if (fd->vsbsize < 0) fd->vsb = 0;
 }
 
-struct f_data *cached_format_html(struct f_data_c *fd, struct object_request *rq, unsigned char *url, struct document_options *opt, int *cch)
+struct f_data *cached_format_html(struct f_data_c *fd, struct object_request *rq, unsigned char *url, struct document_options *opt, int *cch, int report_status)
 {
 	struct session *ses = fd->ses;
 	struct f_data *f;
@@ -1550,8 +1552,10 @@ struct f_data *cached_format_html(struct f_data_c *fd, struct object_request *rq
 			}
 		}
 	}) {};
-	if (ses)
-		print_progress(ses, TEXT_(T_FORMATTING_DOCUMENT));
+	if (ses) {
+		if (report_status || !fd->f_data || fd->f_data->time_to_get >= DISPLAY_FORMATTING_STATUS)
+			print_progress(ses, TEXT_(T_FORMATTING_DOCUMENT));
+	}
 	detach_f_data(&fd->f_data);
 	f = format_html(fd, rq, url, opt, cch);
 	if (f) f->fd = fd;
@@ -1633,14 +1637,13 @@ static void create_new_frames(struct f_data_c *fd, struct frameset_desc *fs, str
 	}
 }
 
-static void html_interpret(struct f_data_c *fd)
+static void html_interpret(struct f_data_c *fd, int report_status)
 {
 	int i;
 	int oxw; int oyw; int oxp; int oyp;
 	struct f_data_c *sf;
 	struct list_head *lsf;
 	int cch;
-	/*int first = !fd->f_data;*/
 	struct document_options o;
 #ifdef JS
 	struct js_event_spec *doc_js;
@@ -1723,7 +1726,7 @@ static void html_interpret(struct f_data_c *fd)
 		for (; i < fd->f_data->nlink_events; i++) copy_js_event_spec(&link_js[i], fd->f_data->link_events[i]);
 	}
 #endif
-	if (!(fd->f_data = cached_format_html(fd, fd->rq, fd->rq->url, &o, &cch))) {
+	if (!(fd->f_data = cached_format_html(fd, fd->rq, fd->rq->url, &o, &cch, report_status))) {
 #ifdef JS
 		for (i = 0; i < nlink_js; i++) free_js_event_spec(link_js[i]);
 		mem_free(link_js);
@@ -1760,14 +1763,13 @@ static void html_interpret(struct f_data_c *fd)
 	}
 
 	d:;
-	/*draw_fd(fd);*/
 }
 
 void html_interpret_recursive(struct f_data_c *f)
 {
 	struct f_data_c *fd;
 	struct list_head *lfd;
-	if (f->rq) html_interpret(f);
+	if (f->rq) html_interpret(f, 1);
 	foreach(struct f_data_c, fd, lfd, f->subframes) html_interpret_recursive(fd);
 }
 
@@ -1776,6 +1778,7 @@ void html_interpret_recursive(struct f_data_c *f)
  */
 struct additional_file *request_additional_file(struct f_data *f, unsigned char *url_)
 {
+	size_t sl;
 	struct additional_file *af;
 	struct list_head *laf;
 	unsigned char *u, *url;
@@ -1793,7 +1796,9 @@ struct additional_file *request_additional_file(struct f_data *f, unsigned char
 		mem_free(url);
 		return af;
 	}
-	af = mem_alloc(sizeof(struct additional_file) + strlen(cast_const_char url));
+	sl = strlen(cast_const_char url);
+	if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
+	af = mem_alloc(sizeof(struct additional_file) + sl);
 	af->use_tag = 0;
 	af->use_tag2 = 0;
 	strcpy(cast_char af->url, cast_const_char url);
@@ -1819,8 +1824,11 @@ static void copy_additional_files(struct additional_files **a)
 	afs->refcount = 1;
 	init_list(afs->af);
 	foreachback(struct additional_file, af, laf, (*a)->af) {
-		struct additional_file *afc = mem_alloc(sizeof(struct additional_file) + strlen(cast_const_char af->url));
-		memcpy(afc, af, sizeof(struct additional_file) + strlen(cast_const_char af->url));
+		struct additional_file *afc;
+		size_t sl = strlen(cast_const_char af->url);
+		if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
+		afc = mem_alloc(sizeof(struct additional_file) + sl);
+		memcpy(afc, af, sizeof(struct additional_file) + sl);
 		if (af->rq) clone_object(af->rq, &afc->rq);
 		add_to_list(afs->af, afc);
 	}
@@ -2023,7 +2031,7 @@ void fd_loaded(struct object_request *rq, void *fd_)
 	}
 	if (fd->rq->state < 0 && (f_is_finished(fd->f_data) || !fd->f_data)) {
 		if (!fd->parsed_done) {
-			html_interpret(fd);
+			html_interpret(fd, 1);
 			if (fd->went_to_position) {
 				if (!fd->goto_position) fd->goto_position = fd->went_to_position, fd->went_to_position = NULL;
 				else mem_free(fd->went_to_position), fd->went_to_position = NULL;
@@ -2051,7 +2059,7 @@ fn:
 	} else if (get_time() - fd->last_update >= fd->next_update_interval || (rq == fd->rq && rq->state < 0)) {
 		uttime t;
 		if (!fd->parsed_done) {
-			html_interpret(fd);
+			html_interpret(fd, rq == fd->rq && rq->state < 0);
 			if (fd->rq->state < 0 && !f_need_reparse(fd->f_data)) {
 				if (fd->went_to_position) {
 					if (!fd->goto_position) fd->goto_position = fd->went_to_position, fd->went_to_position = NULL;
@@ -2344,7 +2352,7 @@ static void continue_download(struct session *ses, unsigned char *file, int mode
 					mem_free(file);
 					goto new_name;
 				}
-				msg_box(ses->term, NULL, TEXT_(T_DOWNLOAD_ERROR), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_COULD_NOT_CREATE_TEMPORARY_FILE), NULL, NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+				msg_box(ses->term, NULL, TEXT_(T_DOWNLOAD_ERROR), AL_CENTER, TEXT_(T_COULD_NOT_CREATE_TEMPORARY_FILE), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 			}
 			mem_free(file);
 			tp_cancel(ses);
@@ -2551,12 +2559,12 @@ static void type_query(struct session *ses, unsigned char *ct, struct assoc *a,
 	}
 	m1 = stracpy(ct);
 	if (!a) {
-		if (!anonymous) msg_box(ses->term, getml(m1, NULL), TEXT_(T_UNKNOWN_TYPE), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_SAVE_OR_DISLPAY_THIS_FILE), NULL, ses, 3, TEXT_(T_SAVE), tp_save, B_ENTER, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
-		else msg_box(ses->term, getml(m1, NULL), TEXT_(T_UNKNOWN_TYPE), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_SAVE_OR_DISLPAY_THIS_FILE), NULL, ses, 2, TEXT_(T_DISPLAY), tp_display, B_ENTER, TEXT_(T_CANCEL), tp_cancel, B_ESC);
+		if (!anonymous) msg_box(ses->term, getml(m1, NULL), TEXT_(T_UNKNOWN_TYPE), AL_CENTER, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_SAVE_OR_DISLPAY_THIS_FILE), MSG_BOX_END, (void *)ses, 3, TEXT_(T_SAVE), tp_save, B_ENTER, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
+		else msg_box(ses->term, getml(m1, NULL), TEXT_(T_UNKNOWN_TYPE), AL_CENTER, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_SAVE_OR_DISLPAY_THIS_FILE), MSG_BOX_END, (void *)ses, 2, TEXT_(T_DISPLAY), tp_display, B_ENTER, TEXT_(T_CANCEL), tp_cancel, B_ESC);
 	} else {
 		m2 = stracpy(a[0].label ? a[0].label : (unsigned char *)"");
-		if (!anonymous) msg_box(ses->term, getml(m1, m2, NULL), TEXT_(T_WHAT_TO_DO), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_OPEN_FILE_WITH), cast_uchar " ", m2, cast_uchar ", ", TEXT_(T_SAVE_IT_OR_DISPLAY_IT), NULL, ses, 4, TEXT_(T_OPEN), tp_open, B_ENTER, TEXT_(T_SAVE), tp_save, 0, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
-		else msg_box(ses->term, getml(m1, m2, NULL), TEXT_(T_WHAT_TO_DO), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_OPEN_FILE_WITH), cast_uchar " ", m2, cast_uchar ", ", TEXT_(T_SAVE_IT_OR_DISPLAY_IT), NULL, ses, 3, TEXT_(T_OPEN), tp_open, B_ENTER, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
+		if (!anonymous) msg_box(ses->term, getml(m1, m2, NULL), TEXT_(T_WHAT_TO_DO), AL_CENTER, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_OPEN_FILE_WITH), cast_uchar " ", m2, cast_uchar ", ", TEXT_(T_SAVE_IT_OR_DISPLAY_IT), MSG_BOX_END, (void *)ses, 4, TEXT_(T_OPEN), tp_open, B_ENTER, TEXT_(T_SAVE), tp_save, 0, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
+		else msg_box(ses->term, getml(m1, m2, NULL), TEXT_(T_WHAT_TO_DO), AL_CENTER, TEXT_(T_CONTENT_TYPE_IS), cast_uchar " ", m1, cast_uchar ".\n", TEXT_(T_DO_YOU_WANT_TO_OPEN_FILE_WITH), cast_uchar " ", m2, cast_uchar ", ", TEXT_(T_SAVE_IT_OR_DISPLAY_IT), MSG_BOX_END, (void *)ses, 3, TEXT_(T_OPEN), tp_open, B_ENTER, TEXT_(T_DISPLAY), tp_display, 0, TEXT_(T_CANCEL), tp_cancel, B_ESC);
 	}
 	if (n) mem_free(a);
 	mem_free(ct);
@@ -2575,14 +2583,14 @@ static void ses_go_to_2nd_state(struct session *ses)
 	}
 	if (ses->tq) {
 		ses_abort_1st_state_loading(ses);
-		if (n) mem_free(a);
+		if (a) mem_free(a);
 		if (ct) mem_free(ct);
 		return;
 	}
 	(ses->tq = ses->rq)->upcall = NULL;
 	ses->rq = NULL;
 	ses_abort_1st_state_loading(ses);
-	if (!n && ses->tq->ce && ses->tq->ce->head) {
+	if (!a && ses->tq->ce && ses->tq->ce->head) {
 		unsigned char *file = get_filename_from_header(ses->tq->ce->head);
 		if (file) {
 			unsigned char *new_ct = get_content_type_by_extension(file);
@@ -2665,7 +2673,7 @@ void goto_url_f(struct session *ses, void (*state2)(struct session *), unsigned
 	ses_destroy_defered_jump(ses);
 	if ((fn = get_external_protocol_function(url))) {
 		if (proxies.only_proxies && url_bypasses_socks(url)) {
-			msg_box(ses->term, NULL, TEXT_(T_ERROR), AL_CENTER, TEXT_(T_NO_PROXY), NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+			msg_box(ses->term, NULL, TEXT_(T_ERROR), AL_CENTER, TEXT_(T_NO_PROXY), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 			return;
 		}
 		fn(ses, url);
@@ -2776,8 +2784,10 @@ void goto_imgmap(struct session *ses, struct f_data_c *fd, unsigned char *url, u
 	goto_url_f(ses, ses_imgmap, url, NULL, fd, -1, 0, 0, 0);
 }
 
-void map_selected(struct terminal *term, struct link_def *ld, struct session *ses)
+void map_selected(struct terminal *term, void *ld_, void *ses_)
 {
+	struct link_def *ld = (struct link_def *)ld_;
+	struct session *ses = (struct session *)ses_;
 	int x = 0;
 	if (ld->onclick) {
 		struct f_data_c *fd = current_frame(ses);
@@ -2881,7 +2891,7 @@ static struct session *create_session(struct window *win)
 	add_to_list(sessions, ses);
 	if (first_use) {
 		first_use = 0;
-		msg_box(term, NULL, TEXT_(T_WELCOME), AL_CENTER | AL_EXTD_TEXT, TEXT_(T_WELCOME_TO_LINKS), cast_uchar "\n\n", TEXT_(T_BASIC_HELP), NULL, NULL, 1, TEXT_(T_OK), NULL, B_ENTER | B_ESC);
+		msg_box(term, NULL, TEXT_(T_WELCOME), AL_CENTER, TEXT_(T_WELCOME_TO_LINKS), cast_uchar "\n\n", TEXT_(T_BASIC_HELP), MSG_BOX_END, NULL, 1, TEXT_(T_OK), msg_box_null, B_ENTER | B_ESC);
 	}
 	return ses;
 }
@@ -2896,18 +2906,21 @@ static struct session *create_session(struct window *win)
  */
 void *create_session_info(int cp, unsigned char *url, unsigned char *framename, int *ll)
 {
-	size_t l = strlen(cast_const_char url);
-	size_t l1 = framename ? strlen(cast_const_char framename) : 0;
-	int *i;
-	if (l > MAXINT || l1 > MAXINT) overalloc();
+	size_t sl = strlen(cast_const_char url);
+	size_t sl1 = framename ? strlen(cast_const_char framename) : 0;
+	int l = (int)sl;
+	int l1 = (int)sl1;
+	unsigned char *i;
+	if (sl > MAXINT || sl1 > MAXINT) overalloc();
 	if (framename && !strcmp(cast_const_char framename, "_blank")) l1 = 0;
-	*ll = (int)(3 * sizeof(int) + l + l1);
-	i = mem_alloc(3 * sizeof(int) + l + l1);
-	i[0] = cp;
-	i[1] = (int)l;
-	i[2] = (int)l1;
-	memcpy(i + 3, url, l);
-	if (l1) memcpy((unsigned char*)(i + 3) + l, framename, l1);
+
+	i = init_str();
+	*ll = 0;
+	add_bytes_to_str(&i, ll, (unsigned char *)&cp, sizeof(int));
+	add_bytes_to_str(&i, ll, (unsigned char *)&l, sizeof(int));
+	add_bytes_to_str(&i, ll, (unsigned char *)&l1, sizeof(int));
+	add_bytes_to_str(&i, ll, url, l);
+	add_bytes_to_str(&i, ll, framename, l1);
 	return i;
 }
 
@@ -3092,7 +3105,9 @@ unsigned char *get_current_title(struct f_data_c *fd, unsigned char *str, size_t
 	if (!fd || !fd->f_data)
 		return NULL;
 
-	return safe_strncpy(str, fd->f_data->title, str_size);
+	safe_strncpy(str, fd->f_data->title, str_size);
+
+	return str;
 }
 
 #if 0
@@ -3119,6 +3134,8 @@ unsigned char *get_current_link_url(struct session *ses, unsigned char *str, siz
 	if (l->type != L_LINK)
 		return NULL;
 
-	return safe_strncpy(str, l->where, str_size);
+	safe_strncpy(str, l->where, str_size);
+
+	return str;
 }
 #endif
