@@ -80,6 +80,7 @@ void smb_func(struct connection *c)
 		}
 		if (c->cache->redirect) mem_free(c->cache->redirect);
 		c->cache->redirect = stracpy(c->url);
+		c->cache->redirect_get = 1;
 		add_to_strn(&c->cache->redirect, cast_uchar "/");
 		c->cache->incomplete = 0;
 		mem_free(host);
@@ -253,7 +254,7 @@ void smb_func(struct connection *c)
 			}
 			break;
 		default:
-			internal_error("unsupported smb client");
+			internal("unsupported smb client");
 		}
 		v[n++] = NULL;
 		EINTRLOOP(rs, execvp(cast_const_char v[0], (void *)v));
@@ -322,7 +323,7 @@ static void smb_read_text(struct connection *c, int sock)
 {
 	int r;
 	struct smb_connection_info *si = c->info;
-	if ((size_t)sizeof(struct smb_connection_info) + si->ntext + page_size + 2 > (size_t)MAXINT) overalloc();
+	if ((unsigned)sizeof(struct smb_connection_info) + si->ntext + page_size + 2 > MAXINT) overalloc();
 	si = mem_realloc(si, sizeof(struct smb_connection_info) + si->ntext + page_size + 2);
 	c->info = si;
 	EINTRLOOP(r, (int)read(sock, si->text + si->ntext, page_size));
@@ -346,7 +347,7 @@ static void smb_read_text(struct connection *c, int sock)
 		int lasti = 0;
 		int i = 0;
 		si->text[si->ntext] = 0;
-		for (i = 0; si->ntext - i > 7; i++) {
+		for (i = 0; i + 7 < si->ntext; i++) {
 			nexti:
 			if ((si->text[i] == '\n' || si->text[i] == '\r') && (si->text[i + 1] == ' ' || (si->text[i + 1] >= '0' && si->text[i + 1] <= '9')) && ((si->text[i + 2] == ' ' && si->text[i + 1] == ' ') || (si->text[i + 2] >= '0' && si->text[i + 2] <= '9')) && (si->text[i + 3] >= '0' && si->text[i + 3] <= '9') && si->text[i + 4] == '%' && si->text[i + 5] == ' ' && si->text[i + 6] == '[') {
 				off_t position, total = 0; /* against warning */
@@ -484,6 +485,7 @@ static void end_smb_connection(struct connection *c)
 		     strstr(cast_const_char si->text, "ERRbadfile") || sdir)) {
 			if (c->cache->redirect) mem_free(c->cache->redirect);
 			c->cache->redirect = stracpy(c->url);
+			c->cache->redirect_get = 1;
 			add_to_strn(&c->cache->redirect, cast_uchar "/");
 			c->cache->incomplete = 0;
 		} else {
@@ -598,7 +600,7 @@ static void end_smb_connection(struct connection *c)
 						int dir;
 						unsigned char *pp;
 						unsigned char *p = ls + 3;
-						while (le2 - p >= 2) {
+						while (p + 2 <= le2) {
 							if (p[0] == ' ' && p[1] == ' ') goto o;
 							p++;
 						}
