@@ -173,15 +173,10 @@ my_strtoll_t my_strtoll(unsigned char *string, unsigned char **end)
 }
 
 /* Copies at most dst_size chars into dst. Ensures null termination of dst. */
-unsigned char *safe_strncpy(unsigned char *dst, const unsigned char *src, size_t dst_size)
+void safe_strncpy(unsigned char *dst, const unsigned char *src, size_t dst_size)
 {
-	size_t to_copy;
-	if (!dst_size) return dst;
-	to_copy = strlen(cast_const_char src);
-	if (to_copy >= dst_size) to_copy = dst_size - 1;
-	memcpy(dst, src, to_copy);
-	memset(dst + to_copy, 0, dst_size - to_copy);
-	return dst;
+	dst[dst_size - 1] = 0;
+	strncpy(cast_char dst, cast_const_char src, dst_size - 1);
 }
 
 #ifdef JS
