@@ -1515,29 +1515,6 @@ static int getscansegment_paged(unsigned char *colors, int x, int y, int length)
 	return 0;
 }
 
-#if 0
-static void svga_draw_bitmaps(struct graphics_device *dev, struct bitmap **hndls, int n
-	,int x, int y)
-{
-	void (*draw_b)(struct graphics_device *, struct bitmap *, int, int);
-
-	TEST_INACTIVITY
-	if (x>=xsize||y>ysize) return;
-	while(x+(*hndls)->x<=0&&n){
-		x+=(*hndls)->x;
-		n--;
-		hndls++;
-	}
-	draw_b = svga_driver.draw_bitmap;
-	while(n&&x<=xsize){
-		draw_b(dev, *hndls, x, y);
-		x+=(*hndls)->x;
-		n--;
-		hndls++;
-	}
-}
-#endif
-
 static void alloc_scroll_buffer(void)
 {
 	if (!scroll_buffer) {
@@ -1710,10 +1687,12 @@ static void svgalib_key_in(struct itrm *p, unsigned char *ev_, int size)
 	if (size != sizeof(struct links_event)) return;
 	if (ev->ev == EV_ABORT) terminate_loop = 1;
 	if (ev->ev != EV_KBD) return;
-	if ((ev->y & KBD_ALT) && ev->x >= '0' && ev->x <= '9') {
+	if (ev->y & KBD_PASTE) goto skip;
+	if ((ev->y & (KBD_CTRL | KBD_ALT)) == KBD_ALT && ev->x >= '0' && ev->x <= '9') {
 		switch_virtual_device((ev->x - '1' + 10) % 10);
 		return;
 	}
+skip:
 	if (g_kbd_codepage(&svga_driver) != utf8_table && ev->x >= 128 && ev->x <= 255)
 		if ((ev->x = cp2u(ev->x, g_kbd_codepage(&svga_driver))) == -1) return;
 	if (current_virtual_device && current_virtual_device->keyboard_handler) current_virtual_device->keyboard_handler(current_virtual_device, ev->x, ev->y);
@@ -2376,26 +2355,6 @@ static unsigned char *svga_init_driver(unsigned char *param, unsigned char *disp
 	return NULL;
 }
 
-/* Return value:	0 alloced on heap
- *			1 alloced in vidram
- *			2 alloced in X server shm
- */
-/*
-static int svga_get_filled_bitmap(struct bitmap *dest, long color)
-{
-	int n;
-
-	if (dest->x && (unsigned)dest->x * (unsigned)dest->y / (unsigned)dest->x != (unsigned)dest->y) overalloc();
-	if ((unsigned)dest->x * (unsigned)dest->y > MAXINT / fb_pixelsize) overalloc();
-	n=dest->x*dest->y*fb_pixelsize;
-	dest->data=mem_alloc(n);
-	pixel_set(dest->data,n,&color);
-	dest->skip=dest->x*fb_pixelsize;
-	dest->flags=0;
-	return 0;
-}
-*/
-
 /* Return value:	0 alloced on heap
  *			1 alloced in vidram
  *			2 alloced in X server shm
@@ -2530,13 +2489,11 @@ struct graphics_driver svga_driver={
 	NULL,
 	NULL,
 	svga_get_empty_bitmap,
-	/*svga_get_filled_bitmap,*/
 	svga_register_bitmap,
 	svga_prepare_strip,
 	svga_commit_strip,
 	svga_unregister_bitmap,
 	NULL, /* svga_draw_bitmap */
-	/*svga_draw_bitmaps,*/
 	NULL, /* get_color */
 	NULL, /* fill_area */
 	NULL, /* draw_hline */
