@@ -159,7 +159,7 @@ unsigned char *get_keepalive_id(unsigned char *url)
 	if (is_proxy_url(url) && !casecmp(d, cast_uchar "https://", 8)) {
 		if (parse_url(d, NULL, NULL, NULL, NULL, NULL, &h, &hl, &p, &pl, NULL, NULL, NULL)) return NULL;
 	}
-	k = p ? p + pl : h + hl;
+	k = p ? p + pl : h ? h + hl : NULL;
 	if (!k) return stracpy(cast_uchar "");
 	return memacpy(url, k - url);
 }
@@ -386,12 +386,30 @@ match:
 	return res;
 }
 
+static unsigned char *rewrite_url_mediawiki_svg(unsigned char *n)
+{
+#ifndef HAVE_SVG
+	const unsigned char u1[] = "/media/math/render/svg/";
+	const unsigned char u2[] = "/media/math/render/png/";
+	unsigned char *d, *s;
+	d = get_url_data(n);
+	if (!d)
+		return n;
+	s = cast_uchar strstr(cast_const_char d, cast_const_char u1);
+	if (!s)
+		return n;
+	memcpy(s, u2, strlen(cast_const_char u2));
+#endif
+	return n;
+}
+
 static unsigned char *rewrite_url(unsigned char *n)
 {
 	extend_str(&n, 1);
 	translate_directories(n);
 	n = translate_hashbang(n);
 	n = rewrite_url_google_docs(n);
+	n = rewrite_url_mediawiki_svg(n);
 	return n;
 }
 
@@ -632,9 +650,7 @@ unsigned char *extract_position(unsigned char *url)
 	if ((u = get_url_data(url))) url = u;
 	if (!(u = cast_uchar strchr(cast_const_char url, POST_CHAR))) u = cast_uchar strchr(cast_const_char url, 0);
 	if (!(uu = memchr(url, '#', u - url))) return NULL;
-	r = mem_alloc(u - uu);
-	memcpy(r, uu + 1, u - uu - 1);
-	r[u - uu - 1] = 0;
+	r = memacpy(uu + 1, u - uu - 1);
 	memmove(uu, u, strlen(cast_const_char u) + 1);
 	return r;
 }
