@@ -214,7 +214,7 @@ void g_text_draw(struct f_data_c *fd, struct g_object *t_, int x, int y)
 				for (j = 0; j < form->rows; j++) {
 					unsigned char *pp = ln->st;
 					int xx = fs->vpos;
-					while (pp < ln->en && xx > 0) {
+					while (ln->st && pp < ln->en && xx > 0) {
 						FWD_UTF_8(pp);
 						xx--;
 					}
@@ -1230,10 +1230,12 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct links_event *ev)
 					}
 					return 1;
 				}
-				if (ev->x == KBD_ENTER) {
+				if (ev->x == KBD_ENTER && !(ev->y & KBD_PASTE)) {
 					return enter(ses, fd, 0);
 				}
 			}
+			if (ev->y & KBD_PASTE)
+				return 0;
 			if (ev->x == KBD_ENTER && fd->f_data->opt.plain == 2) {
 				ses->ds.porn_enable ^= 1;
 				html_interpret_recursive(ses->screen);
@@ -1562,7 +1564,7 @@ void g_find_next(struct f_data_c *f, int a)
 	g_get_search_data(f->f_data);
 	g_get_search(f->f_data, f->ses->search_word);
 	search_word = f->ses->search_word;
-	if (!f->f_data->n_search_positions) msg_box(f->ses->term, NULL, TEXT_(T_SEARCH), AL_CENTER, TEXT_(T_SEARCH_STRING_NOT_FOUND), NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
+	if (!f->f_data->n_search_positions) msg_box(f->ses->term, NULL, TEXT_(T_SEARCH), AL_CENTER, TEXT_(T_SEARCH_STRING_NOT_FOUND), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 
 	highlight_positions = f->f_data->search_positions;
 	highlight_lengths = f->f_data->search_lengths;
