@@ -1565,7 +1565,6 @@ static void x_shutdown_device(struct graphics_device *gd)
 #ifdef X_DEBUG
 	MESSAGE("x_shutdown_device\n");
 #endif
-	if (!gd) return;
 
 	n_wins--;
 	XDestroyWindow(x_display, wi->window);
@@ -1737,14 +1736,14 @@ static long x_get_color(int rgb)
 					return static_color_table[b[0]];
 				return b[0];
 		case 2:		if (x_bitmap_bit_order == LSBFirst)
-					return b[0] | (b[1] << 8);
+					return (unsigned)b[0] | ((unsigned)b[1] << 8);
 				else
-					return b[1] | (b[0] << 8);
-		case 3:		return b[0] | (b[1] << 8) | (b[2] << 16);
+					return (unsigned)b[1] | ((unsigned)b[0] << 8);
+		case 3:		return (unsigned)b[0] | ((unsigned)b[1] << 8) | ((unsigned)b[2] << 16);
 		default:	if (x_bitmap_bit_order == LSBFirst)
-					return b[0] | (b[1] << 8) | (b[2] << 16) | (b[3] << 24);
+					return (unsigned)b[0] | ((unsigned)b[1] << 8) | ((unsigned)b[2] << 16) | ((unsigned)b[3] << 24);
 				else
-					return b[3] | (b[2] << 8) | (b[1] << 16) | (b[0] << 24);
+					return (unsigned)b[3] | ((unsigned)b[2] << 8) | ((unsigned)b[1] << 16) | ((unsigned)b[0] << 24);
 	}
 }
 
@@ -1895,25 +1894,6 @@ static void x_draw_bitmap(struct graphics_device *gd, struct bitmap *bmp, int x,
 }
 
 
-#if 0
-static void x_draw_bitmaps(struct graphics_device *gd, struct bitmap **bmps, int n, int x, int y)
-{
-	int a;
-#ifdef X_DEBUG
-	MESSAGE("x_draw_bitmaps\n");
-#endif
-
-	if (!bmps)return;
-	for (a=0;a<n;a++)
-	{
-		x_draw_bitmap(gd,bmps[a],x,y);
-		x+=(bmps[a])->x;
-	}
-}
-#endif
-
-
-
 static int x_hscroll(struct graphics_device *gd, struct rect_set **set, int sc)
 {
 	XEvent ev;
@@ -2468,13 +2448,11 @@ struct graphics_driver x_driver={
 	NULL,
 	NULL,
 	x_get_empty_bitmap,
-	/*x_get_filled_bitmap,*/
 	x_register_bitmap,
 	x_prepare_strip,
 	x_commit_strip,
 	x_unregister_bitmap,
 	x_draw_bitmap,
-	/*x_draw_bitmaps,*/
 	x_get_color,
 	x_fill_area,
 	x_draw_hline,
